@@ -3,7 +3,7 @@
  * \author Radek Krejci <rkrejci@cesnet.cz>
  * \brief libnetconf2 - NETCONF messages functions
  *
- * Copyright (c) 2015 CESNET, z.s.p.o.
+ * Copyright (c) 2015 - 2021 CESNET, z.s.p.o.
  *
  * This source code is licensed under BSD 3-Clause License (the "License").
  * You may not use this file except in compliance with the License.
@@ -14,9 +14,9 @@
 
 #include <assert.h>
 #include <ctype.h>
+#include <stdarg.h>
 #include <stdlib.h>
 #include <string.h>
-#include <stdarg.h>
 
 #include <libyang/libyang.h>
 
@@ -56,7 +56,10 @@ nc_rpc_act_generic(const struct lyd_node *data, NC_PARAMTYPE paramtype)
     rpc->type = NC_RPC_ACT_GENERIC;
     rpc->has_data = 1;
     if (paramtype == NC_PARAMTYPE_DUP_AND_FREE) {
-        rpc->content.data = lyd_dup(data, 1);
+        if (lyd_dup_single(data, NULL, LYD_DUP_RECURSIVE, &rpc->content.data)) {
+            free(rpc);
+            return NULL;
+        }
     } else {
         rpc->content.data = (struct lyd_node *)data;
     }
@@ -104,7 +107,7 @@ nc_rpc_getconfig(NC_DATASTORE source, const char *filter, NC_WD_MODE wd_mode, NC
     }
 
     if (filter && filter[0] && (filter[0] != '<') && (filter[0] != '/') && !isalpha(filter[0])) {
-        ERR("Filter is neither an XML subtree nor an XPath expression (invalid first char '%c').", filter[0]);
+        ERR(NULL, "Filter is neither an XML subtree nor an XPath expression (invalid first char '%c').", filter[0]);
         return NULL;
     }
 
@@ -129,7 +132,7 @@ nc_rpc_getconfig(NC_DATASTORE source, const char *filter, NC_WD_MODE wd_mode, NC
 
 API struct nc_rpc *
 nc_rpc_edit(NC_DATASTORE target, NC_RPC_EDIT_DFLTOP default_op, NC_RPC_EDIT_TESTOPT test_opt,
-            NC_RPC_EDIT_ERROPT error_opt, const char *edit_content, NC_PARAMTYPE paramtype)
+        NC_RPC_EDIT_ERROPT error_opt, const char *edit_content, NC_PARAMTYPE paramtype)
 {
     struct nc_rpc_edit *rpc;
 
@@ -142,7 +145,7 @@ nc_rpc_edit(NC_DATASTORE target, NC_RPC_EDIT_DFLTOP default_op, NC_RPC_EDIT_TEST
     }
 
     if (edit_content[0] && (edit_content[0] != '<') && !isalpha(edit_content[0])) {
-        ERR("<edit-config> content is neither a URL nor an XML config (invalid first char '%c').", edit_content[0]);
+        ERR(NULL, "<edit-config> content is neither a URL nor an XML config (invalid first char '%c').", edit_content[0]);
         return NULL;
     }
 
@@ -169,7 +172,7 @@ nc_rpc_edit(NC_DATASTORE target, NC_RPC_EDIT_DFLTOP default_op, NC_RPC_EDIT_TEST
 
 API struct nc_rpc *
 nc_rpc_copy(NC_DATASTORE target, const char *url_trg, NC_DATASTORE source, const char *url_or_config_src,
-            NC_WD_MODE wd_mode, NC_PARAMTYPE paramtype)
+        NC_WD_MODE wd_mode, NC_PARAMTYPE paramtype)
 {
     struct nc_rpc_copy *rpc;
 
@@ -182,7 +185,7 @@ nc_rpc_copy(NC_DATASTORE target, const char *url_trg, NC_DATASTORE source, const
     }
 
     if (url_or_config_src && url_or_config_src[0] && (url_or_config_src[0] != '<') && !isalpha(url_or_config_src[0])) {
-        ERR("<copy-config> source is neither a URL nor an XML config (invalid first char '%c').", url_or_config_src[0]);
+        ERR(NULL, "<copy-config> source is neither a URL nor an XML config (invalid first char '%c').", url_or_config_src[0]);
         return NULL;
     }
 
@@ -289,7 +292,7 @@ nc_rpc_get(const char *filter, NC_WD_MODE wd_mode, NC_PARAMTYPE paramtype)
     struct nc_rpc_get *rpc;
 
     if (filter && filter[0] && (filter[0] != '<') && (filter[0] != '/') && !isalpha(filter[0])) {
-        ERR("Filter is neither an XML subtree nor an XPath expression (invalid first char '%c').", filter[0]);
+        ERR(NULL, "Filter is neither an XML subtree nor an XPath expression (invalid first char '%c').", filter[0]);
         return NULL;
     }
 
@@ -335,7 +338,7 @@ nc_rpc_kill(uint32_t session_id)
 
 API struct nc_rpc *
 nc_rpc_commit(int confirmed, uint32_t confirm_timeout, const char *persist, const char *persist_id,
-              NC_PARAMTYPE paramtype)
+        NC_PARAMTYPE paramtype)
 {
     struct nc_rpc_commit *rpc;
 
@@ -412,7 +415,7 @@ nc_rpc_validate(NC_DATASTORE source, const char *url_or_config, NC_PARAMTYPE par
     }
 
     if (url_or_config && url_or_config[0] && (url_or_config[0] != '<') && !isalpha(url_or_config[0])) {
-        ERR("<validate> source is neither a URL nor an XML config (invalid first char '%c').", url_or_config[0]);
+        ERR(NULL, "<validate> source is neither a URL nor an XML config (invalid first char '%c').", url_or_config[0]);
         return NULL;
     }
 
@@ -473,12 +476,12 @@ nc_rpc_getschema(const char *identifier, const char *version, const char *format
 
 API struct nc_rpc *
 nc_rpc_subscribe(const char *stream_name, const char *filter, const char *start_time, const char *stop_time,
-                 NC_PARAMTYPE paramtype)
+        NC_PARAMTYPE paramtype)
 {
     struct nc_rpc_subscribe *rpc;
 
     if (filter && filter[0] && (filter[0] != '<') && (filter[0] != '/') && !isalpha(filter[0])) {
-        ERR("Filter is neither an XML subtree nor an XPath expression (invalid first char '%c').", filter[0]);
+        ERR(NULL, "Filter is neither an XML subtree nor an XPath expression (invalid first char '%c').", filter[0]);
         return NULL;
     }
 
@@ -516,14 +519,14 @@ nc_rpc_subscribe(const char *stream_name, const char *filter, const char *start_
 
 API struct nc_rpc *
 nc_rpc_getdata(const char *datastore, const char *filter, const char *config_filter, char **origin_filter,
-               int origin_filter_count, int negated_origin_filter, uint16_t max_depth, int with_origin,
-               NC_WD_MODE wd_mode, NC_PARAMTYPE paramtype)
+        int origin_filter_count, int negated_origin_filter, uint16_t max_depth, int with_origin, NC_WD_MODE wd_mode,
+        NC_PARAMTYPE paramtype)
 {
     struct nc_rpc_getdata *rpc = NULL;
     int i;
 
     if (filter && filter[0] && (filter[0] != '<') && (filter[0] != '/') && !isalpha(filter[0])) {
-        ERR("Filter is neither an XML subtree nor an XPath expression (invalid first char '%c').", filter[0]);
+        ERR(NULL, "Filter is neither an XML subtree nor an XPath expression (invalid first char '%c').", filter[0]);
         return NULL;
     } else if (!datastore) {
         ERRARG("datastore");
@@ -597,7 +600,7 @@ nc_rpc_editdata(const char *datastore, NC_RPC_EDIT_DFLTOP default_op, const char
     }
 
     if (edit_content[0] && (edit_content[0] != '<') && !isalpha(edit_content[0])) {
-        ERR("<edit-data> content is neither a URL nor an XML config (invalid first char '%c').", edit_content[0]);
+        ERR(NULL, "<edit-data> content is neither a URL nor an XML config (invalid first char '%c').", edit_content[0]);
         return NULL;
     }
 
@@ -624,6 +627,392 @@ nc_rpc_editdata(const char *datastore, NC_RPC_EDIT_DFLTOP default_op, const char
     return (struct nc_rpc *)rpc;
 }
 
+API struct nc_rpc *
+nc_rpc_establishsub(const char *filter, const char *stream_name, const char *start_time,
+        const char *stop_time, const char *encoding, NC_PARAMTYPE paramtype)
+{
+    struct nc_rpc_establishsub *rpc;
+
+    if (!stream_name) {
+        ERRARG("stream_name");
+        return NULL;
+    }
+
+    if (filter && filter[0] && (filter[0] != '<') && (filter[0] != '/') && !isalpha(filter[0])) {
+        ERR(NULL, "Filter is not an XML subtree, an XPath expression, not a filter reference (invalid first char '%c').",
+                filter[0]);
+        return NULL;
+    }
+
+    rpc = malloc(sizeof *rpc);
+    if (!rpc) {
+        ERRMEM;
+        return NULL;
+    }
+
+    rpc->type = NC_RPC_ESTABLISHSUB;
+    if (filter && (paramtype == NC_PARAMTYPE_DUP_AND_FREE)) {
+        rpc->filter = strdup(filter);
+    } else {
+        rpc->filter = (char *)filter;
+    }
+    if (paramtype == NC_PARAMTYPE_DUP_AND_FREE) {
+        rpc->stream = strdup(stream_name);
+    } else {
+        rpc->stream = (char *)stream_name;
+    }
+    if (start_time && (paramtype == NC_PARAMTYPE_DUP_AND_FREE)) {
+        rpc->start = strdup(start_time);
+    } else {
+        rpc->start = (char *)start_time;
+    }
+    if (stop_time && (paramtype == NC_PARAMTYPE_DUP_AND_FREE)) {
+        rpc->stop = strdup(stop_time);
+    } else {
+        rpc->stop = (char *)stop_time;
+    }
+    if (encoding && (paramtype == NC_PARAMTYPE_DUP_AND_FREE)) {
+        rpc->encoding = strdup(encoding);
+    } else {
+        rpc->encoding = (char *)encoding;
+    }
+    rpc->free = (paramtype == NC_PARAMTYPE_CONST ? 0 : 1);
+
+    return (struct nc_rpc *)rpc;
+}
+
+API struct nc_rpc *
+nc_rpc_modifysub(uint32_t id, const char *filter, const char *stop_time, NC_PARAMTYPE paramtype)
+{
+    struct nc_rpc_modifysub *rpc;
+
+    if (!id) {
+        ERRARG("id");
+        return NULL;
+    }
+
+    if (filter && filter[0] && (filter[0] != '<') && (filter[0] != '/') && !isalpha(filter[0])) {
+        ERR(NULL, "Filter is not an XML subtree, an XPath expression, not a filter reference (invalid first char '%c').",
+                filter[0]);
+        return NULL;
+    }
+
+    rpc = malloc(sizeof *rpc);
+    if (!rpc) {
+        ERRMEM;
+        return NULL;
+    }
+
+    rpc->type = NC_RPC_MODIFYSUB;
+    rpc->id = id;
+    if (filter && (paramtype == NC_PARAMTYPE_DUP_AND_FREE)) {
+        rpc->filter = strdup(filter);
+    } else {
+        rpc->filter = (char *)filter;
+    }
+    if (stop_time && (paramtype == NC_PARAMTYPE_DUP_AND_FREE)) {
+        rpc->stop = strdup(stop_time);
+    } else {
+        rpc->stop = (char *)stop_time;
+    }
+    rpc->free = (paramtype == NC_PARAMTYPE_CONST ? 0 : 1);
+
+    return (struct nc_rpc *)rpc;
+}
+
+API struct nc_rpc *
+nc_rpc_deletesub(uint32_t id)
+{
+    struct nc_rpc_deletesub *rpc;
+
+    if (!id) {
+        ERRARG("id");
+        return NULL;
+    }
+
+    rpc = malloc(sizeof *rpc);
+    if (!rpc) {
+        ERRMEM;
+        return NULL;
+    }
+
+    rpc->type = NC_RPC_DELETESUB;
+    rpc->id = id;
+
+    return (struct nc_rpc *)rpc;
+}
+
+API struct nc_rpc *
+nc_rpc_killsub(uint32_t id)
+{
+    struct nc_rpc_killsub *rpc;
+
+    if (!id) {
+        ERRARG("id");
+        return NULL;
+    }
+
+    rpc = malloc(sizeof *rpc);
+    if (!rpc) {
+        ERRMEM;
+        return NULL;
+    }
+
+    rpc->type = NC_RPC_KILLSUB;
+    rpc->id = id;
+
+    return (struct nc_rpc *)rpc;
+}
+
+API struct nc_rpc *
+nc_rpc_establishpush_periodic(const char *datastore, const char *filter, const char *stop_time, const char *encoding,
+        uint32_t period, const char *anchor_time, NC_PARAMTYPE paramtype)
+{
+    struct nc_rpc_establishpush *rpc;
+
+    if (!datastore) {
+        ERRARG("datastore");
+        return NULL;
+    } else if (!period) {
+        ERRARG("period");
+        return NULL;
+    }
+
+    if (filter && filter[0] && (filter[0] != '<') && (filter[0] != '/') && !isalpha(filter[0])) {
+        ERR(NULL, "Filter is not an XML subtree, an XPath expression, not a filter reference (invalid first char '%c').",
+                filter[0]);
+        return NULL;
+    }
+
+    rpc = malloc(sizeof *rpc);
+    if (!rpc) {
+        ERRMEM;
+        return NULL;
+    }
+
+    rpc->type = NC_RPC_ESTABLISHPUSH;
+    if (paramtype == NC_PARAMTYPE_DUP_AND_FREE) {
+        rpc->datastore = strdup(datastore);
+    } else {
+        rpc->datastore = (char *)datastore;
+    }
+    if (filter && (paramtype == NC_PARAMTYPE_DUP_AND_FREE)) {
+        rpc->filter = strdup(filter);
+    } else {
+        rpc->filter = (char *)filter;
+    }
+    if (stop_time && (paramtype == NC_PARAMTYPE_DUP_AND_FREE)) {
+        rpc->stop = strdup(stop_time);
+    } else {
+        rpc->stop = (char *)stop_time;
+    }
+    if (encoding && (paramtype == NC_PARAMTYPE_DUP_AND_FREE)) {
+        rpc->encoding = strdup(encoding);
+    } else {
+        rpc->encoding = (char *)encoding;
+    }
+    rpc->periodic = 1;
+    rpc->period = period;
+    if (anchor_time && (paramtype == NC_PARAMTYPE_DUP_AND_FREE)) {
+        rpc->anchor_time = strdup(anchor_time);
+    } else {
+        rpc->anchor_time = (char *)anchor_time;
+    }
+    rpc->free = (paramtype == NC_PARAMTYPE_CONST ? 0 : 1);
+
+    return (struct nc_rpc *)rpc;
+}
+
+API struct nc_rpc *
+nc_rpc_establishpush_onchange(const char *datastore, const char *filter, const char *stop_time, const char *encoding,
+        uint32_t dampening_period, int sync_on_start, const char **excluded_change, NC_PARAMTYPE paramtype)
+{
+    struct nc_rpc_establishpush *rpc;
+    uint32_t i;
+
+    if (!datastore) {
+        ERRARG("datastore");
+        return NULL;
+    }
+
+    if (filter && filter[0] && (filter[0] != '<') && (filter[0] != '/') && !isalpha(filter[0])) {
+        ERR(NULL, "Filter is not an XML subtree, an XPath expression, not a filter reference (invalid first char '%c').",
+                filter[0]);
+        return NULL;
+    }
+
+    rpc = malloc(sizeof *rpc);
+    if (!rpc) {
+        ERRMEM;
+        return NULL;
+    }
+
+    rpc->type = NC_RPC_ESTABLISHPUSH;
+    if (paramtype == NC_PARAMTYPE_DUP_AND_FREE) {
+        rpc->datastore = strdup(datastore);
+    } else {
+        rpc->datastore = (char *)datastore;
+    }
+    if (filter && (paramtype == NC_PARAMTYPE_DUP_AND_FREE)) {
+        rpc->filter = strdup(filter);
+    } else {
+        rpc->filter = (char *)filter;
+    }
+    if (stop_time && (paramtype == NC_PARAMTYPE_DUP_AND_FREE)) {
+        rpc->stop = strdup(stop_time);
+    } else {
+        rpc->stop = (char *)stop_time;
+    }
+    if (encoding && (paramtype == NC_PARAMTYPE_DUP_AND_FREE)) {
+        rpc->encoding = strdup(encoding);
+    } else {
+        rpc->encoding = (char *)encoding;
+    }
+    rpc->periodic = 0;
+    rpc->dampening_period = dampening_period;
+    rpc->sync_on_start = sync_on_start;
+    if (excluded_change && (paramtype == NC_PARAMTYPE_DUP_AND_FREE)) {
+        rpc->excluded_change = NULL;
+        for (i = 0; excluded_change[i]; ++i) {
+            rpc->excluded_change = realloc(rpc->excluded_change, (i + 2) * sizeof *rpc->excluded_change);
+            rpc->excluded_change[i] = strdup(excluded_change[i]);
+            rpc->excluded_change[i + 1] = NULL;
+        }
+    } else {
+        rpc->excluded_change = (char **)excluded_change;
+    }
+    rpc->free = (paramtype == NC_PARAMTYPE_CONST ? 0 : 1);
+
+    return (struct nc_rpc *)rpc;
+}
+
+API struct nc_rpc *
+nc_rpc_modifypush_periodic(uint32_t id, const char *datastore, const char *filter, const char *stop_time, uint32_t period,
+        const char *anchor_time, NC_PARAMTYPE paramtype)
+{
+    struct nc_rpc_modifypush *rpc;
+
+    if (!id) {
+        ERRARG("id");
+        return NULL;
+    } else if (!datastore) {
+        ERRARG("datastore");
+        return NULL;
+    }
+
+    if (filter && filter[0] && (filter[0] != '<') && (filter[0] != '/') && !isalpha(filter[0])) {
+        ERR(NULL, "Filter is not an XML subtree, an XPath expression, not a filter reference (invalid first char '%c').",
+                filter[0]);
+        return NULL;
+    }
+
+    rpc = malloc(sizeof *rpc);
+    if (!rpc) {
+        ERRMEM;
+        return NULL;
+    }
+
+    rpc->type = NC_RPC_MODIFYPUSH;
+    rpc->id = id;
+    if (paramtype == NC_PARAMTYPE_DUP_AND_FREE) {
+        rpc->datastore = strdup(datastore);
+    } else {
+        rpc->datastore = (char *)datastore;
+    }
+    if (filter && (paramtype == NC_PARAMTYPE_DUP_AND_FREE)) {
+        rpc->filter = strdup(filter);
+    } else {
+        rpc->filter = (char *)filter;
+    }
+    if (stop_time && (paramtype == NC_PARAMTYPE_DUP_AND_FREE)) {
+        rpc->stop = strdup(stop_time);
+    } else {
+        rpc->stop = (char *)stop_time;
+    }
+    rpc->periodic = 1;
+    rpc->period = period;
+    if (anchor_time && (paramtype == NC_PARAMTYPE_DUP_AND_FREE)) {
+        rpc->anchor_time = strdup(anchor_time);
+    } else {
+        rpc->anchor_time = (char *)anchor_time;
+    }
+    rpc->free = (paramtype == NC_PARAMTYPE_CONST ? 0 : 1);
+
+    return (struct nc_rpc *)rpc;
+}
+
+API struct nc_rpc *
+nc_rpc_modifypush_onchange(uint32_t id, const char *datastore, const char *filter, const char *stop_time,
+        uint32_t dampening_period, NC_PARAMTYPE paramtype)
+{
+    struct nc_rpc_modifypush *rpc;
+
+    if (!id) {
+        ERRARG("id");
+        return NULL;
+    } else if (!datastore) {
+        ERRARG("datastore");
+        return NULL;
+    }
+
+    if (filter && filter[0] && (filter[0] != '<') && (filter[0] != '/') && !isalpha(filter[0])) {
+        ERR(NULL, "Filter is not an XML subtree, an XPath expression, not a filter reference (invalid first char '%c').",
+                filter[0]);
+        return NULL;
+    }
+
+    rpc = malloc(sizeof *rpc);
+    if (!rpc) {
+        ERRMEM;
+        return NULL;
+    }
+
+    rpc->type = NC_RPC_MODIFYPUSH;
+    rpc->id = id;
+    if (paramtype == NC_PARAMTYPE_DUP_AND_FREE) {
+        rpc->datastore = strdup(datastore);
+    } else {
+        rpc->datastore = (char *)datastore;
+    }
+    if (filter && (paramtype == NC_PARAMTYPE_DUP_AND_FREE)) {
+        rpc->filter = strdup(filter);
+    } else {
+        rpc->filter = (char *)filter;
+    }
+    if (stop_time && (paramtype == NC_PARAMTYPE_DUP_AND_FREE)) {
+        rpc->stop = strdup(stop_time);
+    } else {
+        rpc->stop = (char *)stop_time;
+    }
+    rpc->periodic = 0;
+    rpc->dampening_period = dampening_period;
+    rpc->free = (paramtype == NC_PARAMTYPE_CONST ? 0 : 1);
+
+    return (struct nc_rpc *)rpc;
+}
+
+API struct nc_rpc *
+nc_rpc_resyncsub(uint32_t id)
+{
+    struct nc_rpc_resyncsub *rpc;
+
+    if (!id) {
+        ERRARG("id");
+        return NULL;
+    }
+
+    rpc = malloc(sizeof *rpc);
+    if (!rpc) {
+        ERRMEM;
+        return NULL;
+    }
+
+    rpc->type = NC_RPC_RESYNCSUB;
+    rpc->id = id;
+
+    return (struct nc_rpc *)rpc;
+}
+
 API void
 nc_rpc_free(struct nc_rpc *rpc)
 {
@@ -640,6 +1029,10 @@ nc_rpc_free(struct nc_rpc *rpc)
     struct nc_rpc_subscribe *rpc_subscribe;
     struct nc_rpc_getdata *rpc_getdata;
     struct nc_rpc_editdata *rpc_editdata;
+    struct nc_rpc_establishsub *rpc_establishsub;
+    struct nc_rpc_modifysub *rpc_modifysub;
+    struct nc_rpc_establishpush *rpc_establishpush;
+    struct nc_rpc_modifypush *rpc_modifypush;
     int i;
 
     if (!rpc) {
@@ -651,7 +1044,7 @@ nc_rpc_free(struct nc_rpc *rpc)
         rpc_generic = (struct nc_rpc_act_generic *)rpc;
         if (rpc_generic->free) {
             if (rpc_generic->has_data) {
-                lyd_free(rpc_generic->content.data);
+                lyd_free_tree(rpc_generic->content.data);
             } else {
                 free(rpc_generic->content.xml_str);
             }
@@ -743,7 +1136,61 @@ nc_rpc_free(struct nc_rpc *rpc)
             free(rpc_editdata->edit_cont);
         }
         break;
-    default:
+    case NC_RPC_ESTABLISHSUB:
+        rpc_establishsub = (struct nc_rpc_establishsub *)rpc;
+        if (rpc_establishsub->free) {
+            free(rpc_establishsub->filter);
+            free(rpc_establishsub->stream);
+            free(rpc_establishsub->start);
+            free(rpc_establishsub->stop);
+            free(rpc_establishsub->encoding);
+        }
+        break;
+    case NC_RPC_MODIFYSUB:
+        rpc_modifysub = (struct nc_rpc_modifysub *)rpc;
+        if (rpc_modifysub->free) {
+            free(rpc_modifysub->filter);
+            free(rpc_modifysub->stop);
+        }
+        break;
+    case NC_RPC_ESTABLISHPUSH:
+        rpc_establishpush = (struct nc_rpc_establishpush *)rpc;
+        if (rpc_establishpush->free) {
+            free(rpc_establishpush->datastore);
+            free(rpc_establishpush->filter);
+            free(rpc_establishpush->stop);
+            free(rpc_establishpush->encoding);
+            if (rpc_establishpush->periodic) {
+                free(rpc_establishpush->anchor_time);
+            } else {
+                if (rpc_establishpush->excluded_change) {
+                    for (i = 0; rpc_establishpush->excluded_change[i]; ++i) {
+                        free(rpc_establishpush->excluded_change[i]);
+                    }
+                    free(rpc_establishpush->excluded_change);
+                }
+            }
+        }
+        break;
+    case NC_RPC_MODIFYPUSH:
+        rpc_modifypush = (struct nc_rpc_modifypush *)rpc;
+        if (rpc_modifypush->free) {
+            free(rpc_modifypush->datastore);
+            free(rpc_modifypush->filter);
+            free(rpc_modifypush->stop);
+            if (rpc_modifypush->periodic) {
+                free(rpc_modifypush->anchor_time);
+            }
+        }
+        break;
+    case NC_RPC_UNKNOWN:
+    case NC_RPC_LOCK:
+    case NC_RPC_UNLOCK:
+    case NC_RPC_KILL:
+    case NC_RPC_DISCARD:
+    case NC_RPC_DELETESUB:
+    case NC_RPC_KILLSUB:
+    case NC_RPC_RESYNCSUB:
         /* nothing special needed */
         break;
     }
@@ -782,57 +1229,6 @@ nc_client_err_clean(struct nc_err *err, struct ly_ctx *ctx)
         lydict_remove(ctx, err->ns[i]);
     }
     free(err->ns);
-    for (i = 0; i < err->other_count; ++i) {
-        lyxml_free(ctx, err->other[i]);
-    }
+    lyd_free_siblings(err->other);
     free(err->other);
 }
-
-API void
-nc_reply_free(struct nc_reply *reply)
-{
-    struct nc_client_reply_error *error;
-    struct nc_reply_data *data;
-    uint32_t i;
-
-    if (!reply) {
-        return;
-    }
-
-    switch (reply->type) {
-    case NC_RPL_DATA:
-        data = (struct nc_reply_data *)reply;
-        lyd_free_withsiblings(data->data);
-        break;
-
-    case NC_RPL_OK:
-        /* nothing to free */
-        break;
-
-    case NC_RPL_ERROR:
-        error = (struct nc_client_reply_error *)reply;
-        for (i = 0; i < error->count; ++i) {
-            nc_client_err_clean(&error->err[i], error->ctx);
-        }
-        free(error->err);
-        break;
-
-    case NC_RPL_NOTIF:
-        nc_notif_free((struct nc_notif *)reply);
-        return;
-    }
-
-    free(reply);
-}
-
-API void
-nc_notif_free(struct nc_notif *notif)
-{
-    if (!notif) {
-        return;
-    }
-
-    lydict_remove(notif->tree->schema->module->ctx, notif->datetime);
-    lyd_free(notif->tree);
-    free(notif);
-}
