@@ -1,10 +1,10 @@
 /**
- * \file messages_client.h
- * \author Radek Krejci <rkrejci@cesnet.cz>
- * \author Michal Vasko <mvasko@cesnet.cz>
- * \brief libnetconf2's public functions and structures of NETCONF client messages.
+ * @file messages_client.h
+ * @author Radek Krejci <rkrejci@cesnet.cz>
+ * @author Michal Vasko <mvasko@cesnet.cz>
+ * @brief libnetconf2's public functions and structures of NETCONF client messages.
  *
- * Copyright (c) 2015 CESNET, z.s.p.o.
+ * Copyright (c) 2015 - 2021 CESNET, z.s.p.o.
  *
  * This source code is licensed under BSD 3-Clause License (the "License").
  * You may not use this file except in compliance with the License.
@@ -64,6 +64,17 @@ typedef enum {
     /* ietf-netconf-nmda */
     NC_RPC_GETDATA,     /**< \<get-data\> RPC. */
     NC_RPC_EDITDATA,    /**< \<edit-data\> RPC. */
+
+    /* ietf-subscribed-notifications */
+    NC_RPC_ESTABLISHSUB,    /**< \<establish-subscription\> RPC. */
+    NC_RPC_MODIFYSUB,       /**< \<modify-subscription\> RPC. */
+    NC_RPC_DELETESUB,       /**< \<delete-subscription\> RPC. */
+    NC_RPC_KILLSUB,         /**< \<kill-subscription\> RPC. */
+
+    /* ietf-yang-push */
+    NC_RPC_ESTABLISHPUSH,   /**< \<establish-subscription\> RPC with augments. */
+    NC_RPC_MODIFYPUSH,      /**< \<modify-subscription\> RPC with augments. */
+    NC_RPC_RESYNCSUB        /**< \<resync-subscription\> RPC. */
 } NC_RPC_TYPE;
 
 /**
@@ -125,8 +136,8 @@ struct nc_err {
     const char **elem;
     /** @brief \<bad-namespace\>, array of the unexpected XML namespaces that caused the error. Part of \<error-info\>. */
     const char **ns;
-    /** @brief Array of the remaining non-standard elements. */
-    struct lyxml_elem **other;
+    /** @brief List of the remaining non-standard opaque nodes. */
+    struct lyd_node *other;
 
     /** @brief Number of items in the attr array */
     uint16_t attr_count;
@@ -139,52 +150,21 @@ struct nc_err {
 };
 
 /**
+ * @struct nc_rpc
  * @brief NETCONF client RPC object
+ *
+ * Note that any stored parameters are not checked for validity because it is performed later,
+ * while sending the RPC via a specific NETCONF session (::nc_send_rpc()) since the NETCONF
+ * capabilities of the session are needed for such a check. An RPC object can be sent via any
+ * NETCONF session which supports all the needed NETCONF capabilities for the RPC.
  */
 struct nc_rpc;
 
-/**
- * @brief NETCONF client rpc-reply object
- */
-struct nc_reply {
-    NC_RPL type; /**< reply type */
-};
-
-/**
- * @brief NETCONF client data rpc-reply object
- */
-struct nc_reply_data {
-    NC_RPL type;            /**< NC_RPL_DATA */
-    struct lyd_node *data;  /**< libyang RPC reply data tree (output of an RPC),
-                                 \<get\> and \<get-config\> replies are special,
-                                 in those cases there is the configuration itself,
-                                 it should be validated as such (using \b LYD_OPT_GET or \b LYD_OPT_GETCONFIG),
-                                 and it can be NULL in a valid reply. */
-};
-
-/**
- * @brief NETCONF client error rpc-reply object
- */
-struct nc_reply_error {
-    NC_RPL type;              /**< NC_RPL_ERROR */
-    const struct nc_err *err; /**< errors, any of the values inside can be NULL */
-    uint32_t count;           /**< number of error structures */
-};
-
-/**
- * @brief NETCONF client notification object
- */
-struct nc_notif {
-    NC_RPL type;           /**< NC_RPL_NOTIF */
-    const char *datetime;  /**< eventTime of the notification */
-    struct lyd_node *tree; /**< libyang data tree of the message */
-};
-
 /**
  * @brief Get the type of the RPC
  *
  * @param[in] rpc RPC to check the type of.
- * @return Type of \p rpc.
+ * @return Type of @p rpc.
  */
 NC_RPC_TYPE nc_rpc_get_type(const struct nc_rpc *rpc);
 
@@ -192,7 +172,9 @@ NC_RPC_TYPE nc_rpc_get_type(const struct nc_rpc *rpc);
  * @brief Create a generic NETCONF RPC or action
  *
  * Note that created object can be sent via any NETCONF session that shares the context
- * of the \p data.
+ * of the @p data.
+ *
+ * @note In case of action, the \<action\> element is added automatically and should not be in @p data.
  *
  * @param[in] data NETCONF RPC data as a data tree.
  * @param[in] paramtype How to further manage data parameters.
@@ -203,11 +185,9 @@ struct nc_rpc *nc_rpc_act_generic(const struct lyd_node *data, NC_PARAMTYPE para
 /**
  * @brief Create a generic NETCONF RPC or action from an XML string
  *
- * Note that functions to create any RPC object do not check validity of the provided
- * parameters. It is checked later while sending the RPC via a specific NETCONF session
- * (#nc_send_rpc()) since the NETCONF capabilities of the session are needed for such a
- * check. Created object can be sent via any NETCONF session which supports all the
- * needed NETCONF capabilities for the RPC.
+ * For details, see ::nc_rpc.
+ *
+ * @note In case of action, the \<action\> element is added automatically and should not be in @p xml_str.
  *
  * @param[in] xml_str NETCONF RPC data as an XML string.
  * @param[in] paramtype How to further manage data parameters.
@@ -218,11 +198,7 @@ struct nc_rpc *nc_rpc_act_generic_xml(const char *xml_str, NC_PARAMTYPE paramtyp
 /**
  * @brief Create NETCONF RPC \<get-config\>
  *
- * Note that functions to create any RPC object do not check validity of the provided
- * parameters. It is checked later while sending the RPC via a specific NETCONF session
- * (#nc_send_rpc()) since the NETCONF capabilities of the session are needed for such a
- * check. Created object can be sent via any NETCONF session which supports all the
- * needed NETCONF capabilities for the RPC.
+ * For details, see ::nc_rpc.
  *
  * @param[in] source Source datastore being queried.
  * @param[in] filter Optional filter data, an XML subtree or XPath expression (with JSON prefixes).
@@ -231,16 +207,12 @@ struct nc_rpc *nc_rpc_act_generic_xml(const char *xml_str, NC_PARAMTYPE paramtyp
  * @return Created RPC object to send via a NETCONF session or NULL in case of (memory allocation) error.
  */
 struct nc_rpc *nc_rpc_getconfig(NC_DATASTORE source, const char *filter, NC_WD_MODE wd_mode,
-                                NC_PARAMTYPE paramtype);
+        NC_PARAMTYPE paramtype);
 
 /**
  * @brief Create NETCONF RPC \<edit-config\>
  *
- * Note that functions to create any RPC object do not check validity of the provided
- * parameters. It is checked later while sending the RPC via a specific NETCONF session
- * (#nc_send_rpc()) since the NETCONF capabilities of the session are needed for such a
- * check. Created object can be sent via any NETCONF session which supports all the
- * needed NETCONF capabilities for the RPC.
+ * For details, see ::nc_rpc.
  *
  * @param[in] target Target datastore being edited.
  * @param[in] default_op Optional default operation.
@@ -251,39 +223,31 @@ struct nc_rpc *nc_rpc_getconfig(NC_DATASTORE source, const char *filter, NC_WD_M
  * @return Created RPC object to send via a NETCONF session or NULL in case of (memory allocation) error.
  */
 struct nc_rpc *nc_rpc_edit(NC_DATASTORE target, NC_RPC_EDIT_DFLTOP default_op, NC_RPC_EDIT_TESTOPT test_opt,
-                           NC_RPC_EDIT_ERROPT error_opt, const char *edit_content, NC_PARAMTYPE paramtype);
+        NC_RPC_EDIT_ERROPT error_opt, const char *edit_content, NC_PARAMTYPE paramtype);
 
 /**
  * @brief Create NETCONF RPC \<copy-config\>
  *
- * Note that functions to create any RPC object do not check validity of the provided
- * parameters. It is checked later while sending the RPC via a specific NETCONF session
- * (#nc_send_rpc()) since the NETCONF capabilities of the session are needed for such a
- * check. Created object can be sent via any NETCONF session which supports all the
- * needed NETCONF capabilities for the RPC.
+ * For details, see ::nc_rpc.
  *
  * @param[in] target Target datastore.
- * @param[in] url_trg Used instead \p target if the target is an URL.
+ * @param[in] url_trg Used instead @p target if the target is an URL.
  * @param[in] source Source datastore.
- * @param[in] url_or_config_src Used instead \p source if the source is an URL or a config.
+ * @param[in] url_or_config_src Used instead @p source if the source is an URL or a config.
  * @param[in] wd_mode Optional with-defaults capability mode.
  * @param[in] paramtype How to further manage data parameters.
  * @return Created RPC object to send via a NETCONF session or NULL in case of (memory allocation) error.
  */
 struct nc_rpc *nc_rpc_copy(NC_DATASTORE target, const char *url_trg, NC_DATASTORE source,
-                           const char *url_or_config_src, NC_WD_MODE wd_mode, NC_PARAMTYPE paramtype);
+        const char *url_or_config_src, NC_WD_MODE wd_mode, NC_PARAMTYPE paramtype);
 
 /**
  * @brief Create NETCONF RPC \<delete-config\>
  *
- * Note that functions to create any RPC object do not check validity of the provided
- * parameters. It is checked later while sending the RPC via a specific NETCONF session
- * (#nc_send_rpc()) since the NETCONF capabilities of the session are needed for such a
- * check. Created object can be sent via any NETCONF session which supports all the
- * needed NETCONF capabilities for the RPC.
+ * For details, see ::nc_rpc.
  *
  * @param[in] target Target datastore to delete.
- * @param[in] url Used instead \p target if the target is an URL.
+ * @param[in] url Used instead @p target if the target is an URL.
  * @param[in] paramtype How to further manage data parameters.
  * @return Created RPC object to send via a NETCONF session or NULL in case of (memory allocation) error.
  */
@@ -292,11 +256,7 @@ struct nc_rpc *nc_rpc_delete(NC_DATASTORE target, const char *url, NC_PARAMTYPE
 /**
  * @brief Create NETCONF RPC \<lock\>
  *
- * Note that functions to create any RPC object do not check validity of the provided
- * parameters. It is checked later while sending the RPC via a specific NETCONF session
- * (#nc_send_rpc()) since the NETCONF capabilities of the session are needed for such a
- * check. Created object can be sent via any NETCONF session which supports all the
- * needed NETCONF capabilities for the RPC.
+ * For details, see ::nc_rpc.
  *
  * @param[in] target Target datastore of the operation.
  * @return Created RPC object to send via a NETCONF session or NULL in case of (memory allocation) error.
@@ -306,11 +266,7 @@ struct nc_rpc *nc_rpc_lock(NC_DATASTORE target);
 /**
  * @brief Create NETCONF RPC \<unlock\>
  *
- * Note that functions to create any RPC object do not check validity of the provided
- * parameters. It is checked later while sending the RPC via a specific NETCONF session
- * (#nc_send_rpc()) since the NETCONF capabilities of the session are needed for such a
- * check. Created object can be sent via any NETCONF session which supports all the
- * needed NETCONF capabilities for the RPC.
+ * For details, see ::nc_rpc.
  *
  * @param[in] target Target datastore of the operation.
  * @return Created RPC object to send via a NETCONF session or NULL in case of (memory allocation) error.
@@ -320,11 +276,7 @@ struct nc_rpc *nc_rpc_unlock(NC_DATASTORE target);
 /**
  * @brief Create NETCONF RPC \<get\>
  *
- * Note that functions to create any RPC object do not check validity of the provided
- * parameters. It is checked later while sending the RPC via a specific NETCONF session
- * (#nc_send_rpc()) since the NETCONF capabilities of the session are needed for such a
- * check. Created object can be sent via any NETCONF session which supports all the
- * needed NETCONF capabilities for the RPC.
+ * For details, see ::nc_rpc.
  *
  * @param[in] filter Optional filter data, an XML subtree or XPath expression (with JSON prefixes).
  * @param[in] wd_mode Optional with-defaults capability mode.
@@ -336,11 +288,7 @@ struct nc_rpc *nc_rpc_get(const char *filter, NC_WD_MODE wd_mode, NC_PARAMTYPE p
 /**
  * @brief Create NETCONF RPC \<kill-session\>
  *
- * Note that functions to create any RPC object do not check validity of the provided
- * parameters. It is checked later while sending the RPC via a specific NETCONF session
- * (#nc_send_rpc()) since the NETCONF capabilities of the session are needed for such a
- * check. Created object can be sent via any NETCONF session which supports all the
- * needed NETCONF capabilities for the RPC.
+ * For details, see ::nc_rpc.
  *
  * @param[in] session_id Session ID of the session to kill.
  * @return Created RPC object to send via a NETCONF session or NULL in case of (memory allocation) error.
@@ -350,11 +298,7 @@ struct nc_rpc *nc_rpc_kill(uint32_t session_id);
 /**
  * @brief Create NETCONF RPC \<commit\>
  *
- * Note that functions to create any RPC object do not check validity of the provided
- * parameters. It is checked later while sending the RPC via a specific NETCONF session
- * (#nc_send_rpc()) since the NETCONF capabilities of the session are needed for such a
- * check. Created object can be sent via any NETCONF session which supports all the
- * needed NETCONF capabilities for the RPC.
+ * For details, see ::nc_rpc.
  *
  * @param[in] confirmed Whether the commit is to be confirmed.
  * @param[in] confirm_timeout Optional confirm timeout.
@@ -364,16 +308,12 @@ struct nc_rpc *nc_rpc_kill(uint32_t session_id);
  * @return Created RPC object to send via a NETCONF session or NULL in case of (memory allocation) error.
  */
 struct nc_rpc *nc_rpc_commit(int confirmed, uint32_t confirm_timeout, const char *persist, const char *persist_id,
-                             NC_PARAMTYPE paramtype);
+        NC_PARAMTYPE paramtype);
 
 /**
  * @brief Create NETCONF RPC \<discard-changes\>
  *
- * Note that functions to create any RPC object do not check validity of the provided
- * parameters. It is checked later while sending the RPC via a specific NETCONF session
- * (#nc_send_rpc()) since the NETCONF capabilities of the session are needed for such a
- * check. Created object can be sent via any NETCONF session which supports all the
- * needed NETCONF capabilities for the RPC.
+ * For details, see ::nc_rpc.
  *
  * @return Created RPC object to send via a NETCONF session or NULL in case of (memory allocation) error.
  */
@@ -382,11 +322,7 @@ struct nc_rpc *nc_rpc_discard(void);
 /**
  * @brief Create NETCONF RPC \<cancel-commit\>
  *
- * Note that functions to create any RPC object do not check validity of the provided
- * parameters. It is checked later while sending the RPC via a specific NETCONF session
- * (#nc_send_rpc()) since the NETCONF capabilities of the session are needed for such a
- * check. Created object can be sent via any NETCONF session which supports all the
- * needed NETCONF capabilities for the RPC.
+ * For details, see ::nc_rpc.
  *
  * @param[in] persist_id Optional identification string of a persistent confirmed commit.
  * @param[in] paramtype How to further manage data parameters.
@@ -397,14 +333,10 @@ struct nc_rpc *nc_rpc_cancel(const char *persist_id, NC_PARAMTYPE paramtype);
 /**
  * @brief Create NETCONF RPC \<validate\>
  *
- * Note that functions to create any RPC object do not check validity of the provided
- * parameters. It is checked later while sending the RPC via a specific NETCONF session
- * (#nc_send_rpc()) since the NETCONF capabilities of the session are needed for such a
- * check. Created object can be sent via any NETCONF session which supports all the
- * needed NETCONF capabilities for the RPC.
+ * For details, see ::nc_rpc.
  *
  * @param[in] source Source datastore being validated.
- * @param[in] url_or_config Used instead \p source if the source is an URL or a config.
+ * @param[in] url_or_config Used instead @p source if the source is an URL or a config.
  * @param[in] paramtype How to further manage data parameters.
  * @return Created RPC object to send via a NETCONF session or NULL in case of (memory allocation) error.
  */
@@ -413,11 +345,7 @@ struct nc_rpc *nc_rpc_validate(NC_DATASTORE source, const char *url_or_config, N
 /**
  * @brief Create NETCONF RPC \<get-schema\>
  *
- * Note that functions to create any RPC object do not check validity of the provided
- * parameters. It is checked later while sending the RPC via a specific NETCONF session
- * (#nc_send_rpc()) since the NETCONF capabilities of the session are needed for such a
- * check. Created object can be sent via any NETCONF session which supports all the
- * needed NETCONF capabilities for the RPC.
+ * For details, see ::nc_rpc.
  *
  * @param[in] identifier Requested model identifier.
  * @param[in] version Optional model version, either YANG version (1.0/1.1) or revision date.
@@ -430,11 +358,7 @@ struct nc_rpc *nc_rpc_getschema(const char *identifier, const char *version, con
 /**
  * @brief Create NETCONF RPC \<create-subscription\>
  *
- * Note that functions to create any RPC object do not check validity of the provided
- * parameters. It is checked later while sending the RPC via a specific NETCONF session
- * (#nc_send_rpc()) since the NETCONF capabilities of the session are needed for such a
- * check. Created object can be sent via any NETCONF session which supports all the
- * needed NETCONF capabilities for the RPC.
+ * For details, see ::nc_rpc.
  *
  * @param[in] stream_name Optional name of a NETCONF stream to subscribe to.
  * @param[in] filter Optional filter data, an XML subtree or XPath expression (with JSON prefixes).
@@ -444,22 +368,18 @@ struct nc_rpc *nc_rpc_getschema(const char *identifier, const char *version, con
  * @return Created RPC object to send via a NETCONF session or NULL in case of (memory allocation) error.
  */
 struct nc_rpc *nc_rpc_subscribe(const char *stream_name, const char *filter, const char *start_time,
-                                const char *stop_time, NC_PARAMTYPE paramtype);
+        const char *stop_time, NC_PARAMTYPE paramtype);
 
 /**
  * @brief Create NETCONF RPC \<get-data\>
  *
- * Note that functions to create any RPC object do not check validity of the provided
- * parameters. It is checked later while sending the RPC via a specific NETCONF session
- * (#nc_send_rpc()) since the NETCONF capabilities of the session are needed for such a
- * check. Created object can be sent via any NETCONF session which supports all the
- * needed NETCONF capabilities for the RPC.
+ * For details, see ::nc_rpc.
  *
  * @param[in] datastore Source datastore, foreign identity so a module name prefix is required.
  * @param[in] filter Optional filter data, an XML subtree or XPath expression (with JSON prefixes).
  * @param[in] config_filter Optional config filter, "true" for config-only data, "false" for state-only data.
  * @param[in] origin_filter Optional origin filter array, selects only nodes of this or derived origin.
- * @param[in] origin_filter_count Count of filters is \p origin_filter.
+ * @param[in] origin_filter_count Count of filters is @p origin_filter.
  * @param[in] neg_origin_filter Whether origin filters are negated or not.
  * @param[in] max_depth Maximum depth of returned subtrees, 0 for unlimited.
  * @param[in] with_origin Whether return data origin.
@@ -468,17 +388,13 @@ struct nc_rpc *nc_rpc_subscribe(const char *stream_name, const char *filter, con
  * @return Created RPC object to send via a NETCONF session or NULL in case of (memory allocation) error.
  */
 struct nc_rpc *nc_rpc_getdata(const char *datastore, const char *filter, const char *config_filter, char **origin_filter,
-                              int origin_filter_count, int neg_origin_filter, uint16_t max_depth, int with_origin,
-                              NC_WD_MODE wd_mode, NC_PARAMTYPE paramtype);
+        int origin_filter_count, int neg_origin_filter, uint16_t max_depth, int with_origin, NC_WD_MODE wd_mode,
+        NC_PARAMTYPE paramtype);
 
 /**
  * @brief Create NETCONF RPC \<get-data\>
  *
- * Note that functions to create any RPC object do not check validity of the provided
- * parameters. It is checked later while sending the RPC via a specific NETCONF session
- * (#nc_send_rpc()) since the NETCONF capabilities of the session are needed for such a
- * check. Created object can be sent via any NETCONF session which supports all the
- * needed NETCONF capabilities for the RPC.
+ * For details, see ::nc_rpc.
  *
  * @param[in] datastore Source datastore, foreign identity so a module name prefix is required.
  * @param[in] default_op Optional default operation.
@@ -487,7 +403,141 @@ struct nc_rpc *nc_rpc_getdata(const char *datastore, const char *filter, const c
  * @return Created RPC object to send via a NETCONF session or NULL in case of (memory allocation) error.
  */
 struct nc_rpc *nc_rpc_editdata(const char *datastore, NC_RPC_EDIT_DFLTOP default_op, const char *edit_content,
-                               NC_PARAMTYPE paramtype);
+        NC_PARAMTYPE paramtype);
+
+/**
+ * @brief Create NETCONF RPC \<establish-subscription\>
+ *
+ * For details, see ::nc_rpc.
+ *
+ * @param[in] filter Optional filter data, an XML subtree, XPath expression (with JSON prefixes),
+ * or filter reference, selected based on the first character.
+ * @param[in] stream_name Name of a NETCONF stream to subscribe to.
+ * @param[in] start_time Optional YANG datetime identifying the start of the subscription.
+ * @param[in] stop_time Optional YANG datetime identifying the end of the subscription.
+ * @param[in] encoding Optional specific encoding to use.
+ * @param[in] paramtype How to further manage data parameters.
+ * @return Created RPC object to send via a NETCONF session or NULL in case of (memory allocation) error.
+ */
+struct nc_rpc *nc_rpc_establishsub(const char *filter, const char *stream_name, const char *start_time,
+        const char *stop_time, const char *encoding, NC_PARAMTYPE paramtype);
+
+/**
+ * @brief Create NETCONF RPC \<modify-subscription\>
+ *
+ * For details, see ::nc_rpc.
+ *
+ * @param[in] id Subscription ID to modify.
+ * @param[in] filter Optional new filter data, an XML subtree, XPath expression (with JSON prefixes),
+ * or filter reference, selected based on the first character.
+ * @param[in] stop_time Optional new YANG datetime identifying the end of the subscription.
+ * @param[in] paramtype How to further manage data parameters.
+ * @return Created RPC object to send via a NETCONF session or NULL in case of (memory allocation) error.
+ */
+struct nc_rpc *nc_rpc_modifysub(uint32_t id, const char *filter, const char *stop_time, NC_PARAMTYPE paramtype);
+
+/**
+ * @brief Create NETCONF RPC \<delete-subscription\>
+ *
+ * For details, see ::nc_rpc.
+ *
+ * @param[in] id Subscription ID to delete.
+ * @return Created RPC object to send via a NETCONF session or NULL in case of (memory allocation) error.
+ */
+struct nc_rpc *nc_rpc_deletesub(uint32_t id);
+
+/**
+ * @brief Create NETCONF RPC \<kill-subscription\>
+ *
+ * For details, see ::nc_rpc.
+ *
+ * @param[in] id Subscription ID to kill.
+ * @return Created RPC object to send via a NETCONF session or NULL in case of (memory allocation) error.
+ */
+struct nc_rpc *nc_rpc_killsub(uint32_t id);
+
+/**
+ * @brief Create NETCONF RPC \<establish-subscription\> with augments from ietf-yang-push for a periodic subscription
+ *
+ * For details, see ::nc_rpc.
+ *
+ * @param[in] datastore Source datastore, foreign identity so a module name prefix is required.
+ * @param[in] filter Optional filter data, an XML subtree, XPath expression (with JSON prefixes),
+ * or filter reference, selected based on the first character.
+ * @param[in] stop_time Optional YANG datetime identifying the end of the subscription.
+ * @param[in] encoding Optional specific encoding to use.
+ * @param[in] period Subscription period in centiseconds (0.01s).
+ * @param[in] anchor_time Optional anchor datetime for the period.
+ * @param[in] paramtype How to further manage data parameters.
+ * @return Created RPC object to send via a NETCONF session or NULL in case of (memory allocation) error.
+ */
+struct nc_rpc *nc_rpc_establishpush_periodic(const char *datastore, const char *filter, const char *stop_time,
+        const char *encoding, uint32_t period, const char *anchor_time, NC_PARAMTYPE paramtype);
+
+/**
+ * @brief Create NETCONF RPC \<establish-subscription\> with augments from ietf-yang-push for an on-change subscription
+ *
+ * For details, see ::nc_rpc.
+ *
+ * @param[in] datastore Source datastore, foreign identity so a module name prefix is required.
+ * @param[in] filter Optional filter data, an XML subtree, XPath expression (with JSON prefixes),
+ * or filter reference, selected based on the first character.
+ * @param[in] stop_time Optional YANG datetime identifying the end of the subscription.
+ * @param[in] encoding Optional specific encoding to use.
+ * @param[in] dampening_period Optional dampening period of the notifications.
+ * @param[in] sync_on_start Whether to send a full push-update notification on subscription start.
+ * @param[in] excluded_change Optional NULL-terminated array of excluded changes.
+ * @param[in] paramtype How to further manage data parameters.
+ * @return Created RPC object to send via a NETCONF session or NULL in case of (memory allocation) error.
+ */
+struct nc_rpc *nc_rpc_establishpush_onchange(const char *datastore, const char *filter, const char *stop_time,
+        const char *encoding, uint32_t dampening_period, int sync_on_start, const char **excluded_change,
+        NC_PARAMTYPE paramtype);
+
+/**
+ * @brief Create NETCONF RPC \<modify-subscription\> with augments from ietf-yang-push for a periodic subscription
+ *
+ * For details, see ::nc_rpc.
+ *
+ * @param[in] id Subscription ID to modify.
+ * @param[in] datastore Source datastore, foreign identity so a module name prefix is required.
+ * @param[in] filter Optional filter data, an XML subtree, XPath expression (with JSON prefixes),
+ * or filter reference, selected based on the first character.
+ * @param[in] stop_time Optional YANG datetime identifying the end of the subscription.
+ * @param[in] period Subscription period in centiseconds (0.01s).
+ * @param[in] anchor_time Optional anchor datetime for the period.
+ * @param[in] paramtype How to further manage data parameters.
+ * @return Created RPC object to send via a NETCONF session or NULL in case of (memory allocation) error.
+ */
+struct nc_rpc *nc_rpc_modifypush_periodic(uint32_t id, const char *datastore, const char *filter, const char *stop_time,
+        uint32_t period, const char *anchor_time, NC_PARAMTYPE paramtype);
+
+/**
+ * @brief Create NETCONF RPC \<modify-subscription\> with augments from ietf-yang-push for an on-change subscription
+ *
+ * For details, see ::nc_rpc.
+ *
+ * @param[in] id Subscription ID to modify.
+ * @param[in] datastore Source datastore, foreign identity so a module name prefix is required.
+ * @param[in] filter Optional filter data, an XML subtree, XPath expression (with JSON prefixes),
+ * or filter reference, selected based on the first character.
+ * @param[in] stop_time Optional YANG datetime identifying the end of the subscription.
+ * @param[in] dampening_period Optional dampening period of the notifications.
+ * @param[in] paramtype How to further manage data parameters.
+ * @return Created RPC object to send via a NETCONF session or NULL in case of (memory allocation) error.
+ */
+struct nc_rpc *nc_rpc_modifypush_onchange(uint32_t id, const char *datastore, const char *filter, const char *stop_time,
+        uint32_t dampening_period, NC_PARAMTYPE paramtype);
+
+/**
+ * @brief Create NETCONF RPC \<resync-subscription\>
+ *
+ * For details, see ::nc_rpc.
+ *
+ * @param[in] id Subscription ID to resync.
+ * @return Created RPC object to send via a NETCONF session or NULL in case of (memory allocation) error.
+ */
+struct nc_rpc *nc_rpc_resyncsub(uint32_t id);
 
 /**
  * @brief Free the NETCONF RPC object.
@@ -496,21 +546,7 @@ struct nc_rpc *nc_rpc_editdata(const char *datastore, NC_RPC_EDIT_DFLTOP default
  */
 void nc_rpc_free(struct nc_rpc *rpc);
 
-/**
- * @brief Free the NETCONF RPC reply object.
- *
- * @param[in] reply Object to free.
- */
-void nc_reply_free(struct nc_reply *reply);
-
-/**
- * @brief Free the NETCONF Notification object.
- *
- * @param[in] notif Object to free.
- */
-void nc_notif_free(struct nc_notif *notif);
-
-/**@} Client Messages */
+/** @} Client Messages */
 
 #ifdef __cplusplus
 }
