@@ -1,9 +1,9 @@
 /**
- * \file session_client.h
- * \author Michal Vasko <mvasko@cesnet.cz>
- * \brief libnetconf2 session client manipulation
+ * @file session_client.h
+ * @author Michal Vasko <mvasko@cesnet.cz>
+ * @brief libnetconf2 session client manipulation
  *
- * Copyright (c) 2015 CESNET, z.s.p.o.
+ * Copyright (c) 2015 - 2021 CESNET, z.s.p.o.
  *
  * This source code is licensed under BSD 3-Clause License (the "License").
  * You may not use this file except in compliance with the License.
@@ -22,20 +22,16 @@ extern "C" {
 #include <libyang/libyang.h>
 
 #ifdef NC_ENABLED_SSH
-
-#include <libssh/libssh.h>
-
+# include <libssh/libssh.h>
 #endif
 
 #ifdef NC_ENABLED_TLS
-
-#include <openssl/ssl.h>
-
+# include <openssl/ssl.h>
 #endif
 
-#include "session.h"
-#include "netconf.h"
 #include "messages_client.h"
+#include "netconf.h"
+#include "session.h"
 
 /**
  * @addtogroup client
@@ -70,7 +66,7 @@ const char *nc_client_get_schema_searchpath(void);
  * @brief Set callback function to get missing schemas.
  *
  * @param[in] clb Callback responsible for returning the missing model.
- * @param[in] user_data Arbitrary data that will always be passed to the callback \p clb.
+ * @param[in] user_data Arbitrary data that will always be passed to the callback @p clb.
  * @return 0 on success, 1 on (memory allocation) failure.
  */
 int nc_client_set_schema_callback(ly_module_imp_clb clb, void *user_data);
@@ -119,7 +115,7 @@ void nc_client_init(void);
  */
 void nc_client_destroy(void);
 
-/**@} Client */
+/** @} Client */
 
 /**
  * @defgroup client_session Client Session
@@ -168,7 +164,7 @@ struct nc_session *nc_connect_inout(int fdin, int fdout, struct ly_ctx *ctx);
  */
 struct nc_session *nc_connect_unix(const char *address, struct ly_ctx *ctx);
 
-/**@} Client Session */
+/** @} Client Session */
 
 #ifdef NC_ENABLED_SSH
 
@@ -192,7 +188,7 @@ struct nc_session *nc_connect_unix(const char *address, struct ly_ctx *ctx);
  * @param[in] priv Optional private data to be passed to the callback function.
  */
 void nc_client_ssh_set_auth_hostkey_check_clb(int (*auth_hostkey_check)(const char *hostname, ssh_session session, void *priv),
-                                              void *priv);
+        void *priv);
 
 /**
  * @brief Get currently set SSH authentication hostkey check (knownhosts) callback and its private data previously set
@@ -202,7 +198,7 @@ void nc_client_ssh_set_auth_hostkey_check_clb(int (*auth_hostkey_check)(const ch
  * @param[out] priv Currently set (optional) private data to be passed to the callback function.
  */
 void nc_client_ssh_get_auth_hostkey_check_clb(int (**auth_hostkey_check)(const char *hostname, ssh_session session, void *priv),
-                                              void **priv);
+        void **priv);
 
 /**
  * @brief Set SSH password authentication callback.
@@ -216,7 +212,7 @@ void nc_client_ssh_get_auth_hostkey_check_clb(int (**auth_hostkey_check)(const c
  * @param[in] priv Optional private data to be passed to the callback function.
  */
 void nc_client_ssh_set_auth_password_clb(char *(*auth_password)(const char *username, const char *hostname, void *priv),
-                                         void *priv);
+        void *priv);
 
 /**
  * @brief Get currently set SSH password authentication callback and its private data previously set
@@ -226,7 +222,7 @@ void nc_client_ssh_set_auth_password_clb(char *(*auth_password)(const char *user
  * @param[out] priv Currently set (optional) private data to be passed to the callback function.
  */
 void nc_client_ssh_get_auth_password_clb(char *(**auth_password)(const char *username, const char *hostname, void *priv),
-                                         void **priv);
+        void **priv);
 
 /**
  * @brief Set SSH interactive authentication callback.
@@ -241,8 +237,8 @@ void nc_client_ssh_get_auth_password_clb(char *(**auth_password)(const char *use
  * @param[in] priv Optional private data to be passed to the callback function.
  */
 void nc_client_ssh_set_auth_interactive_clb(char *(*auth_interactive)(const char *auth_name, const char *instruction,
-                                                                      const char *prompt, int echo, void *priv),
-                                            void *priv);
+        const char *prompt, int echo, void *priv),
+        void *priv);
 
 /**
  * @brief Get currently set SSH interactive authentication callback and its private data previously set
@@ -252,8 +248,8 @@ void nc_client_ssh_set_auth_interactive_clb(char *(*auth_interactive)(const char
  * @param[out] priv Currently set (optional) private data to be passed to the callback function.
  */
 void nc_client_ssh_get_auth_interactive_clb(char *(**auth_interactive)(const char *auth_name, const char *instruction,
-                                                                       const char *prompt, int echo, void *priv),
-                                            void **priv);
+        const char *prompt, int echo, void *priv),
+        void **priv);
 
 /**
  * @brief Set SSH publickey authentication encrypted private key passphrase callback.
@@ -267,7 +263,7 @@ void nc_client_ssh_get_auth_interactive_clb(char *(**auth_interactive)(const cha
  * @param[in] priv Optional private data to be passed to the callback function.
  */
 void nc_client_ssh_set_auth_privkey_passphrase_clb(char *(*auth_privkey_passphrase)(const char *privkey_path, void *priv),
-                                                   void *priv);
+        void *priv);
 
 /**
  * @brief Get currently set SSH publickey authentication encrypted private key passphrase callback and its private data
@@ -277,7 +273,7 @@ void nc_client_ssh_set_auth_privkey_passphrase_clb(char *(*auth_privkey_passphra
  * @param[out] priv Currently set (optional) private data to be passed to the callback function.
  */
 void nc_client_ssh_get_auth_privkey_passphrase_clb(char *(**auth_privkey_passphrase)(const char *privkey_path, void *priv),
-                                                   void **priv);
+        void **priv);
 
 /**
  * @brief Add an SSH public and private key pair to be used for client authentication.
@@ -324,7 +320,7 @@ int nc_client_ssh_get_keypair(int idx, const char **pub_key, const char **priv_k
  * - public key authentication (1)
  *
  * @param[in] auth_type Authentication method to modify the preference of.
- * @param[in] pref Preference of \p auth_type. Higher number increases priority, negative values disable the method.
+ * @param[in] pref Preference of @p auth_type. Higher number increases priority, negative values disable the method.
  */
 void nc_client_ssh_set_auth_pref(NC_SSH_AUTH_TYPE auth_type, int16_t pref);
 
@@ -332,7 +328,7 @@ void nc_client_ssh_set_auth_pref(NC_SSH_AUTH_TYPE auth_type, int16_t pref);
  * @brief Get SSH authentication method preference.
  *
  * @param[in] auth_type Authentication method to retrieve the prefrence of.
- * @return Preference of the \p auth_type.
+ * @return Preference of the @p auth_type.
  */
 int16_t nc_client_ssh_get_auth_pref(NC_SSH_AUTH_TYPE auth_type);
 
@@ -376,7 +372,7 @@ struct nc_session *nc_connect_ssh(const char *host, uint16_t port, struct ly_ctx
  *
  * SSH session can have any options set, they will not be modified. If no options were set,
  * host 'localhost', port 22, and the username detected from the EUID is used. If socket is
- * set and connected only the host and the username must be set/is detected. Or the \p ssh_session
+ * set and connected only the host and the username must be set/is detected. Or the @p ssh_session
  * can already be authenticated in which case it is used directly.
  *
  * @param[in] ssh_session libssh structure representing SSH session object. After passing it
@@ -408,7 +404,7 @@ struct nc_session *nc_connect_libssh(ssh_session ssh_session, struct ly_ctx *ctx
  */
 struct nc_session *nc_connect_ssh_channel(struct nc_session *session, struct ly_ctx *ctx);
 
-/**@} Client SSH */
+/** @} Client SSH */
 
 #endif /* NC_ENABLED_SSH */
 
@@ -426,8 +422,8 @@ struct nc_session *nc_connect_ssh_channel(struct nc_session *session, struct ly_
  * @brief Set client authentication identity - a certificate and a private key.
  *
  * @param[in] client_cert Path to the file containing the client certificate.
- * @param[in] client_key Path to the file containing the private key for the \p client_cert.
- *                       If NULL, key is expected to be stored with \p client_cert.
+ * @param[in] client_key Path to the file containing the private key for the @p client_cert.
+ *                       If NULL, key is expected to be stored with @p client_cert.
  * @return 0 on success, -1 on error.
  */
 int nc_client_tls_set_cert_key_paths(const char *client_cert, const char *client_key);
@@ -436,7 +432,7 @@ int nc_client_tls_set_cert_key_paths(const char *client_cert, const char *client
  * @brief Get client authentication identity - a certificate and a private key.
  *
  * @param[out] client_cert Path to the file containing the client certificate. Can be NULL.
- * @param[out] client_key Path to the file containing the private key for the \p client_cert. Can be NULL.
+ * @param[out] client_key Path to the file containing the private key for the @p client_cert. Can be NULL.
  */
 void nc_client_tls_get_cert_key_paths(const char **client_cert, const char **client_key);
 
@@ -515,7 +511,7 @@ struct nc_session *nc_connect_tls(const char *host, uint16_t port, struct ly_ctx
  */
 struct nc_session *nc_connect_libssl(SSL *tls, struct ly_ctx *ctx);
 
-/**@} Client TLS */
+/** @} Client TLS */
 
 #endif /* NC_ENABLED_TLS */
 
@@ -528,7 +524,7 @@ struct nc_session *nc_connect_libssl(SSL *tls, struct ly_ctx *ctx);
  * @brief Get session capabilities.
  *
  * @param[in] session Session to get the information from.
- * @return NULL-terminated array of the \p session capabilities.
+ * @return NULL-terminated array of the @p session capabilities.
  */
 const char * const *nc_session_get_cpblts(const struct nc_session *session);
 
@@ -552,29 +548,25 @@ int nc_session_ntf_thread_running(const struct nc_session *session);
 /**
  * @brief Receive NETCONF RPC reply.
  *
- * Be careful, normally there is a whole RPC reply (output) of an RPC in the \p reply.
- * However, if a reply to \<get\> or \<get-config\> RPC is received, the \p reply is
- * actually the configuration (with either state data or not). This means, for example,
- * that the reply data in these cases should not be validated with \b LYD_OPT_RPCREPLY,
- * but \b LYD_OPT_GET and \b LYD_OPT_GETCONFIG, respectively.
+ * @note This function can be called in a single thread only.
  *
  * @param[in] session NETCONF session from which the function gets data. It must be the
- *            client side session object.
+ * client side session object.
  * @param[in] rpc Original RPC this should be the reply to.
  * @param[in] msgid Expected message ID of the reply.
  * @param[in] timeout Timeout for reading in milliseconds. Use negative value for infinite
- *            waiting and 0 for immediate return if data are not available on the wire.
- * @param[in] parseroptions libyang parseroptions flags, do not set the data type, it is set
- *            internally. \b LYD_OPT_DESTRUCT and \b LYD_OPT_NOSIBLINGS is ignored.
- * @param[out] reply Resulting object of NETCONF RPC reply. Set only on #NC_MSG_REPLY and #NC_MSG_REPLY_ERR_MSGID return.
+ * waiting and 0 for immediate return if data are not available on the wire.
+ * @param[out] envp NETCONF rpc-reply XML envelopes.
+ * @param[out] op Parsed NETCONF reply data, if any (none for \<ok\> or error replies). Set only on #NC_MSG_REPLY
+ * and #NC_MSG_REPLY_ERR_MSGID return.
  * @return #NC_MSG_REPLY for success,
- *         #NC_MSG_WOULDBLOCK if \p timeout has elapsed,
+ *         #NC_MSG_WOULDBLOCK if @p timeout has elapsed,
  *         #NC_MSG_ERROR if reading has failed,
  *         #NC_MSG_NOTIF if a notification was read instead (call this function again to get the reply), and
  *         #NC_MSG_REPLY_ERR_MSGID if a reply with missing or wrong message-id was received.
  */
 NC_MSG_TYPE nc_recv_reply(struct nc_session *session, struct nc_rpc *rpc, uint64_t msgid, int timeout,
-                          int parseroptions, struct nc_reply **reply);
+        struct lyd_node **envp, struct lyd_node **op);
 
 /**
  * @brief Receive NETCONF Notification.
@@ -583,13 +575,14 @@ NC_MSG_TYPE nc_recv_reply(struct nc_session *session, struct nc_rpc *rpc, uint64
  *            client side session object.
  * @param[in] timeout Timeout for reading in milliseconds. Use negative value for infinite
  *            waiting and 0 for immediate return if data are not available on the wire.
- * @param[out] notif Resulting object of NETCONF Notification.
+ * @param[out] envp NETCONF notification XML envelopes.
+ * @param[out] op Parsed NETCONF notification data.
  * @return #NC_MSG_NOTIF for success,
- *         #NC_MSG_WOULDBLOCK if \p timeout has elapsed,
+ *         #NC_MSG_WOULDBLOCK if @p timeout has elapsed,
  *         #NC_MSG_ERROR if reading has failed, and
  *         #NC_MSG_REPLY if a reply was read instead (call this function again to get a notification).
  */
-NC_MSG_TYPE nc_recv_notif(struct nc_session* session, int timeout, struct nc_notif **notif);
+NC_MSG_TYPE nc_recv_notif(struct nc_session *session, int timeout, struct lyd_node **envp, struct lyd_node **op);
 
 /**
  * @brief Receive NETCONF Notifications in a separate thread until the session is terminated
@@ -598,18 +591,17 @@ NC_MSG_TYPE nc_recv_notif(struct nc_session* session, int timeout, struct nc_not
  * @param[in] session Netconf session to read notifications from.
  * @param[in] notif_clb Function that is called for every received notification (including
  *            \<notificationComplete\>). Parameters are the session the notification was received on
- *            and the notification itself.
+ *            and the notification data.
  * @return 0 if the thread was successfully created, -1 on error.
  */
 int nc_recv_notif_dispatch(struct nc_session *session,
-                           void (*notif_clb)(struct nc_session *session, const struct nc_notif *notif));
+        void (*notif_clb)(struct nc_session *session, const struct lyd_node *envp, const struct lyd_node *op));
 
 /**
  * @brief Send NETCONF RPC message via the session.
  *
  * @param[in] session NETCONF session where the RPC will be written.
- * @param[in] rpc NETCOFN RPC object to send via specified session. Object can be created by
- *            nc_rpc_lock(), nc_rpc_unlock() and nc_rpc_generic() functions.
+ * @param[in] rpc NETCONF RPC object to send via the specified session.
  * @param[in] timeout Timeout for writing in milliseconds. Use negative value for infinite
  *            waiting and 0 for return if data cannot be sent immediately.
  * @param[out] msgid If RPC was successfully sent, this is it's message ID.
@@ -630,7 +622,7 @@ NC_MSG_TYPE nc_send_rpc(struct nc_session *session, struct nc_rpc *rpc, int time
  */
 void nc_client_session_set_not_strict(struct nc_session *session);
 
-/**@} Client Session */
+/** @} Client Session */
 
 #ifdef __cplusplus
 }
