@@ -1,10 +1,10 @@
 /**
- * \file session_p.h
- * \author Radek Krejci <rkrejci@cesnet.cz>
- * \author Michal Vasko <mvasko@cesnet.cz>
- * \brief libnetconf2 session manipulation
+ * @file session_p.h
+ * @author Radek Krejci <rkrejci@cesnet.cz>
+ * @author Michal Vasko <mvasko@cesnet.cz>
+ * @brief libnetconf2 session manipulation
  *
- * Copyright (c) 2017 CESNET, z.s.p.o.
+ * Copyright (c) 2017 - 2021 CESNET, z.s.p.o.
  *
  * This source code is licensed under BSD 3-Clause License (the "License").
  * You may not use this file except in compliance with the License.
@@ -16,27 +16,28 @@
 #ifndef NC_SESSION_PRIVATE_H_
 #define NC_SESSION_PRIVATE_H_
 
-#include <stdint.h>
 #include <pthread.h>
+#include <stdint.h>
 #include <sys/stat.h>
 
 #include <libyang/libyang.h>
 
+#include "compat.h"
 #include "libnetconf.h"
+#include "messages_client.h"
 #include "netconf.h"
 #include "session.h"
-#include "messages_client.h"
 
 #ifdef NC_ENABLED_SSH
 
-#   include <libssh/libssh.h>
-#   include <libssh/callbacks.h>
-#   include <libssh/server.h>
+# include <libssh/callbacks.h>
+# include <libssh/libssh.h>
+# include <libssh/server.h>
 
 /* seconds */
-#   define NC_SSH_TIMEOUT 10
+# define NC_SSH_TIMEOUT 10
 /* number of all supported authentication methods */
-#   define NC_SSH_AUTH_COUNT 3
+# define NC_SSH_AUTH_COUNT 3
 
 /* ACCESS unlocked */
 struct nc_client_ssh_opts {
@@ -84,8 +85,8 @@ struct nc_server_ssh_opts {
 
 #ifdef NC_ENABLED_TLS
 
-#   include <openssl/bio.h>
-#   include <openssl/ssl.h>
+# include <openssl/bio.h>
+# include <openssl/ssl.h>
 
 /* ACCESS unlocked */
 struct nc_client_tls_opts {
@@ -175,8 +176,11 @@ struct nc_server_opts {
     /* ACCESS unlocked */
     NC_WD_MODE wd_basic_mode;
     int wd_also_supported;
-    unsigned int capabilities_count;
+    uint32_t capabilities_count;
     const char **capabilities;
+    char *(*content_id_clb)(void *user_data);
+    void *content_id_data;
+    void (*content_id_data_free)(void *data);
 
     /* ACCESS unlocked */
     uint16_t hello_timeout;
@@ -197,7 +201,7 @@ struct nc_server_opts {
 #ifdef NC_ENABLED_TLS
     int (*user_verify_clb)(const struct nc_session *session);
 
-    int (*server_cert_clb)(const char *name, void *user_data, char **cert_path, char **cert_data,char **privkey_path,
+    int (*server_cert_clb)(const char *name, void *user_data, char **cert_path, char **cert_data, char **privkey_path,
             char **privkey_data, NC_SSH_KEY_TYPE *privkey_type);
     void *server_cert_data;
     void (*server_cert_data_free)(void *data);
@@ -291,12 +295,12 @@ struct nc_server_opts {
     pthread_rwlock_t ch_client_lock;
 
     /* Atomic IDs */
-    ATOMIC_UINT32_T new_session_id;
-    ATOMIC_UINT32_T new_client_id;
+    ATOMIC_T new_session_id;
+    ATOMIC_T new_client_id;
 };
 
 /**
- * Sleep time in msec to wait between nc_recv_notif() calls.
+ * Sleep time in usec to wait between nc_recv_notif() calls.
  */
 #define NC_CLIENT_NOTIF_THREAD_SLEEP 10000
 
@@ -372,10 +376,11 @@ typedef enum {
 #define NC_VERSION_10_ENDTAG_LEN 6
 
 /**
- * @brief Container to serialize PRC messages
+ * @brief Container to serialize RPC messages
  */
 struct nc_msg_cont {
-    struct lyxml_elem *msg;
+    struct ly_in *msg;
+    NC_MSG_TYPE type;         /**< can be either NC_MSG_REPLY or NC_MSG_NOTIF */
     struct nc_msg_cont *next;
 };
 
@@ -394,8 +399,8 @@ struct nc_session {
 
     /* Transport implementation */
     NC_TRANSPORT_IMPL ti_type;   /**< transport implementation type to select items from ti union */
-    pthread_mutex_t *io_lock;    /**< input/output lock, note that in case of libssh TI, it will be shared
-                                      with other NETCONF sessions on the same SSH session (but different SSH channel) */
+    pthread_mutex_t *io_lock;    /**< input/output lock, note that in case of libssh TI, it will be shared with
+                                      other NETCONF sessions on the same SSH session (but different SSH channel) */
 
     union {
         struct {
@@ -435,9 +440,10 @@ struct nc_session {
             /* client side only data */
             uint64_t msgid;
             char **cpblts;                 /**< list of server's capabilities on client side */
-            struct nc_msg_cont *replies;   /**< queue for RPC replies received instead of notifications */
-            struct nc_msg_cont *notifs;    /**< queue for notifications received instead of RPC reply */
-            ATOMIC_PTR ntf_tid;            /**< running notifications receiving thread */
+            pthread_mutex_t msgs_lock;     /**< lock for the msgs buffer */
+            struct nc_msg_cont *msgs;      /**< queue for messages received of different type than expected */
+            ATOMIC_T ntf_thread;           /**< flag whether notification thread for this session is running or not,
+                                                2 means it should quit */
 
             /* client flags */
             /* some server modules failed to load so the data from them will be ignored - not use strict flag for parsing */
@@ -447,7 +453,7 @@ struct nc_session {
             /* server side only data */
             time_t session_start;          /**< real time the session was created */
             time_t last_rpc;               /**< monotonic time (seconds) the last RPC was received on this session */
-            int ntf_status;                /**< flag whether the session is subscribed to any stream */
+            int ntf_status;                /**< flag (count) whether the session is subscribed to notifications */
 
             pthread_mutex_t rpc_lock;    /**< lock indicating RPC processing, this lock is always locked before io_lock!! */
             pthread_cond_t rpc_cond;     /**< RPC condition (tied with rpc_lock and rpc_inuse) */
@@ -503,16 +509,14 @@ struct nc_pollsession {
 
 struct nc_ntf_thread_arg {
     struct nc_session *session;
-    void (*notif_clb)(struct nc_session *session, const struct nc_notif *notif);
+    void (*notif_clb)(struct nc_session *session, const struct lyd_node *envp, const struct lyd_node *op);
 };
 
 void *nc_realloc(void *ptr, size_t size);
 
-NC_MSG_TYPE nc_send_msg_io(struct nc_session *session, int io_timeout, struct lyd_node *op);
+struct passwd *nc_getpwuid(uid_t uid, struct passwd *pwd_buf, char **buf, size_t *buf_size);
 
-#ifndef HAVE_PTHREAD_MUTEX_TIMEDLOCK
-int pthread_mutex_timedlock(pthread_mutex_t *mutex, const struct timespec *abstime);
-#endif
+NC_MSG_TYPE nc_send_msg_io(struct nc_session *session, int io_timeout, struct lyd_node *op);
 
 int nc_gettimespec_mono(struct timespec *ts);
 
@@ -532,16 +536,56 @@ int nc_session_rpc_lock(struct nc_session *session, int timeout, const char *fun
 
 int nc_session_rpc_unlock(struct nc_session *session, int timeout, const char *func);
 
+/**
+ * @brief Lock IO lock on a session.
+ *
+ * @param[in] session Session to lock.
+ * @param[in] timeout Timeout in msec to use.
+ * @param[in] func Caller function for logging.
+ * @return 1 on success;
+ * @return 0 on timeout;
+ * @return -1 on error.
+ */
 int nc_session_io_lock(struct nc_session *session, int timeout, const char *func);
 
+/**
+ * @brief Unlock IO lock on a session.
+ *
+ * @param[in] session Session to unlock.
+ * @param[in] func Caller function for logging.
+ * @return 1 on success;
+ * @return -1 on error.
+ */
 int nc_session_io_unlock(struct nc_session *session, const char *func);
 
+/**
+ * @brief Lock MSGS lock on a session.
+ *
+ * @param[in] session Session to lock.
+ * @param[in,out] timeout Timeout in msec to use. If positive and on successful lock, is updated based on what was elapsed.
+ * @param[in] func Caller function for logging.
+ * @return 1 on success;
+ * @return 0 on timeout;
+ * @return -1 on error.
+ */
+int nc_session_client_msgs_lock(struct nc_session *session, int *timeout, const char *func);
+
+/**
+ * @brief Unlock MSGS lock on a session.
+ *
+ * @param[in] session Session to unlock.
+ * @param[in] func Caller function for logging.
+ * @return 1 on success;
+ * @return -1 on error.
+ */
+int nc_session_client_msgs_unlock(struct nc_session *session, const char *func);
+
 int nc_ps_lock(struct nc_pollsession *ps, uint8_t *id, const char *func);
 
 int nc_ps_unlock(struct nc_pollsession *ps, uint8_t id, const char *func);
 
 /**
- * @brief Fill libyang context in \p session. Context models are based on the stored session
+ * @brief Fill libyang context in @p session. Context models are based on the stored session
  *        capabilities. If the server does not support \<get-schema\>, the models are searched
  *        for in the directory set using nc_client_schema_searchpath().
  *
@@ -551,7 +595,7 @@ int nc_ps_unlock(struct nc_pollsession *ps, uint8_t id, const char *func);
 int nc_ctx_check_and_fill(struct nc_session *session);
 
 /**
- * @brief Perform NETCONF handshake on \p session.
+ * @brief Perform NETCONF handshake on @p session.
  *
  * @param[in] session NETCONF session to use.
  * @return NC_MSG_HELLO on success, NC_MSG_BAD_HELLO on client \<hello\> message parsing fail
@@ -606,7 +650,7 @@ int nc_sock_listen_unix(const char *address, const struct nc_server_unix_opts *o
  * @brief Accept a new connection on a listening socket.
  *
  * @param[in] binds Structure with the listening sockets.
- * @param[in] bind_count Number of \p binds.
+ * @param[in] bind_count Number of @p binds.
  * @param[in] timeout Timeout for accepting.
  * @param[out] host Host of the remote peer. Can be NULL.
  * @param[out] port Port of the new connection. Can be NULL.
@@ -709,7 +753,7 @@ int nc_accept_ssh_session(struct nc_session *session, int sock, int timeout);
  *
  * @param[in] sshsession SSH session the message arrived on.
  * @param[in] msg SSH message itself.
- * @param[in] data NETCONF session running on \p sshsession.
+ * @param[in] data NETCONF session running on @p sshsession.
  * @return 0 if the message was handled, 1 if it is left up to libssh.
  */
 int nc_sshcb_msg(ssh_session sshsession, ssh_message msg, void *data);
@@ -756,30 +800,29 @@ void _nc_client_tls_destroy_opts(struct nc_client_tls_opts *opts);
  * @param[in] session NETCONF session from which the message is being read.
  * @param[in] io_timeout Timeout in milliseconds. Negative value means infinite timeout,
  *            zero value causes to return immediately.
- * @param[out] data XML tree built from the read data.
- * @return Type of the read message. #NC_MSG_WOULDBLOCK is returned if timeout is positive
- * (or zero) value and it passed out without any data on the wire. #NC_MSG_ERROR is
- * returned on error and #NC_MSG_NONE is never returned by this function.
+ * @param[out] msg Input handled with the NETCONF message (application layer data).
+ * @return 1 on success.
+ * @return 0 on timeout.
+ * @return -1 on error.
+ * @return -2 on malformed message error.
  */
-NC_MSG_TYPE nc_read_msg_poll_io(struct nc_session* session, int io_timeout, struct lyxml_elem **data);
+int nc_read_msg_poll_io(struct nc_session *session, int io_timeout, struct ly_in **msg);
 
 /**
- * @brief Read message from the wire.
- *
- * Accepts hello, rpc, rpc-reply and notification. Received string is transformed into
- * libyang XML tree and the message type is detected from the top level element.
+ * @brief Read a message from the wire.
  *
  * @param[in] session NETCONF session from which the message is being read.
  * @param[in] io_timeout Timeout in milliseconds. Negative value means infinite timeout,
  *            zero value causes to return immediately.
- * @param[out] data XML tree built from the read data.
- * @param[in] passing_io_lock True if \p session IO lock is already held. This function always unlocks
+ * @param[out] msg Input handled with the NETCONF message (application layer data).
+ * @param[in] passing_io_lock True if @p session IO lock is already held. This function always unlocks
  *            it before returning!
- * @return Type of the read message. #NC_MSG_WOULDBLOCK is returned if timeout is positive
- * (or zero) value and it passed out without any data on the wire. #NC_MSG_ERROR is
- * returned on error and #NC_MSG_NONE is never returned by this function.
+ * @return 1 on success.
+ * @return 0 on timeout.
+ * @return -1 on error.
+ * @return -2 on malformed message error.
  */
-NC_MSG_TYPE nc_read_msg_io(struct nc_session* session, int io_timeout, struct lyxml_elem **data, int passing_io_lock);
+int nc_read_msg_io(struct nc_session *session, int io_timeout, struct ly_in **msg, int passing_io_lock);
 
 /**
  * @brief Write message into wire.
@@ -791,12 +834,9 @@ NC_MSG_TYPE nc_read_msg_io(struct nc_session* session, int io_timeout, struct ly
  * specific additional parameters are required or accepted:
  * - #NC_MSG_RPC
  *   - `struct lyd_node *op;` - operation (content of the \<rpc/\> to be sent. Required parameter.
- *   - `const char *attrs;` - additional attributes to be added into the \<rpc/\> element.
- *     Required parameter.
- *     `message-id` attribute is added automatically and default namespace is set to #NC_NS_BASE.
- *     Optional parameter.
+ *   - `const char *attrs;` - additional attributes to be added into the \<rpc/\> element. Required parameter.
  * - #NC_MSG_REPLY
- *   - `struct lyxml_node *rpc_elem;` - root of the RPC object to reply to. Required parameter.
+ *   - `struct lyd_node_opaq *rpc_envp;` - parsed envelopes of the RPC to reply to. Required parameter.
  *   - `struct nc_server_reply *reply;` - RPC reply. Required parameter.
  * - #NC_MSG_NOTIF
  *   - `struct nc_server_notif *notif;` - notification object. Required parameter.
