@@ -23,21 +23,22 @@
     #include <crypt.h>
 #endif
 
+#include <errno.h>
+#include <fcntl.h>
+#include <pwd.h>
 #include <stdlib.h>
 #include <string.h>
-#include <sys/types.h>
 #include <sys/stat.h>
-#include <pwd.h>
-#include <errno.h>
+#include <sys/types.h>
 #include <time.h>
-#include <fcntl.h>
 #include <unistd.h>
 
+#include "compat.h"
+#include "libnetconf.h"
 #include "session_server.h"
 #include "session_server_ch.h"
-#include "libnetconf.h"
 
-#if !defined(HAVE_CRYPT_R)
+#if !defined (HAVE_CRYPT_R)
 pthread_mutex_t crypt_lock = PTHREAD_MUTEX_INITIALIZER;
 #endif
 
@@ -131,7 +132,7 @@ nc_server_ssh_add_hostkey(const char *name, int16_t idx, struct nc_server_ssh_op
     if (idx != opts->hostkey_count - 1) {
         memmove(opts->hostkeys + idx + 1, opts->hostkeys + idx, opts->hostkey_count - idx);
     }
-    opts->hostkeys[idx] = lydict_insert(server_opts.ctx, name, 0);
+    lydict_insert(server_opts.ctx, name, 0, &opts->hostkeys[idx]);
 
     return 0;
 }
@@ -147,7 +148,9 @@ nc_server_ssh_endpt_add_hostkey(const char *endpt_name, const char *name, int16_
     if (!endpt) {
         return -1;
     }
+
     ret = nc_server_ssh_add_hostkey(name, idx, endpt->opts.ssh);
+
     /* UNLOCK */
     pthread_rwlock_unlock(&server_opts.endpt_lock);
 
@@ -156,7 +159,7 @@ nc_server_ssh_endpt_add_hostkey(const char *endpt_name, const char *name, int16_
 
 API void
 nc_server_ssh_set_passwd_auth_clb(int (*passwd_auth_clb)(const struct nc_session *session, const char *password, void *user_data),
-                                  void *user_data, void (*free_user_data)(void *user_data))
+        void *user_data, void (*free_user_data)(void *user_data))
 {
     server_opts.passwd_auth_clb = passwd_auth_clb;
     server_opts.passwd_auth_data = user_data;
@@ -165,7 +168,7 @@ nc_server_ssh_set_passwd_auth_clb(int (*passwd_auth_clb)(const struct nc_session
 
 API void
 nc_server_ssh_set_interactive_auth_clb(int (*interactive_auth_clb)(const struct nc_session *session, ssh_message msg, void *user_data),
-                                  void *user_data, void (*free_user_data)(void *user_data))
+        void *user_data, void (*free_user_data)(void *user_data))
 {
     server_opts.interactive_auth_clb = interactive_auth_clb;
     server_opts.interactive_auth_data = user_data;
@@ -174,7 +177,7 @@ nc_server_ssh_set_interactive_auth_clb(int (*interactive_auth_clb)(const struct
 
 API void
 nc_server_ssh_set_pubkey_auth_clb(int (*pubkey_auth_clb)(const struct nc_session *session, ssh_key key, void *user_data),
-                                  void *user_data, void (*free_user_data)(void *user_data))
+        void *user_data, void (*free_user_data)(void *user_data))
 {
     server_opts.pubkey_auth_clb = pubkey_auth_clb;
     server_opts.pubkey_auth_data = user_data;
@@ -193,7 +196,9 @@ nc_server_ssh_ch_client_endpt_add_hostkey(const char *client_name, const char *e
     if (!endpt) {
         return -1;
     }
+
     ret = nc_server_ssh_add_hostkey(name, idx, endpt->opts.ssh);
+
     /* UNLOCK */
     nc_server_ch_client_unlock(client);
 
@@ -270,7 +275,9 @@ nc_server_ssh_endpt_del_hostkey(const char *endpt_name, const char *name, int16_
     if (!endpt) {
         return -1;
     }
+
     ret = nc_server_ssh_del_hostkey(name, idx, endpt->opts.ssh);
+
     /* UNLOCK */
     pthread_rwlock_unlock(&server_opts.endpt_lock);
 
@@ -289,7 +296,9 @@ nc_server_ssh_ch_client_endpt_del_hostkey(const char *client_name, const char *e
     if (!endpt) {
         return -1;
     }
+
     ret = nc_server_ssh_del_hostkey(name, idx, endpt->opts.ssh);
+
     /* UNLOCK */
     nc_server_ch_client_unlock(client);
 
@@ -359,7 +368,9 @@ nc_server_ssh_endpt_mov_hostkey(const char *endpt_name, const char *key_mov, con
     if (!endpt) {
         return -1;
     }
+
     ret = nc_server_ssh_mov_hostkey(key_mov, key_after, endpt->opts.ssh);
+
     /* UNLOCK */
     pthread_rwlock_unlock(&server_opts.endpt_lock);
 
@@ -379,7 +390,9 @@ nc_server_ssh_ch_client_endpt_mov_hostkey(const char *client_name, const char *e
     if (!endpt) {
         return -1;
     }
+
     ret = nc_server_ssh_mov_hostkey(key_mov, key_after, endpt->opts.ssh);
+
     /* UNLOCK */
     nc_server_ch_client_unlock(client);
 
@@ -404,7 +417,9 @@ nc_server_ssh_endpt_set_auth_methods(const char *endpt_name, int auth_methods)
     if (!endpt) {
         return -1;
     }
+
     ret = nc_server_ssh_set_auth_methods(auth_methods, endpt->opts.ssh);
+
     /* UNLOCK */
     pthread_rwlock_unlock(&server_opts.endpt_lock);
 
@@ -423,7 +438,9 @@ nc_server_ssh_ch_client_endpt_set_auth_methods(const char *client_name, const ch
     if (!endpt) {
         return -1;
     }
+
     ret = nc_server_ssh_set_auth_methods(auth_methods, endpt->opts.ssh);
+
     /* UNLOCK */
     nc_server_ch_client_unlock(client);
 
@@ -441,7 +458,9 @@ nc_server_ssh_endpt_get_auth_methods(const char *endpt_name)
     if (!endpt) {
         return -1;
     }
+
     ret = endpt->opts.ssh->auth_methods;
+
     /* UNLOCK */
     pthread_rwlock_unlock(&server_opts.endpt_lock);
 
@@ -460,7 +479,9 @@ nc_server_ssh_ch_client_endpt_get_auth_methods(const char *client_name, const ch
     if (!endpt) {
         return -1;
     }
+
     ret = endpt->opts.ssh->auth_methods;
+
     /* UNLOCK */
     nc_server_ch_client_unlock(client);
 
@@ -490,7 +511,9 @@ nc_server_ssh_endpt_set_auth_attempts(const char *endpt_name, uint16_t auth_atte
     if (!endpt) {
         return -1;
     }
+
     ret = nc_server_ssh_set_auth_attempts(auth_attempts, endpt->opts.ssh);
+
     /* UNLOCK */
     pthread_rwlock_unlock(&server_opts.endpt_lock);
 
@@ -509,7 +532,9 @@ nc_server_ssh_ch_client_endpt_set_auth_attempts(const char *client_name, const c
     if (!endpt) {
         return -1;
     }
+
     ret = nc_server_ssh_set_auth_attempts(auth_attempts, endpt->opts.ssh);
+
     /* UNLOCK */
     nc_server_ch_client_unlock(client);
 
@@ -539,7 +564,9 @@ nc_server_ssh_endpt_set_auth_timeout(const char *endpt_name, uint16_t auth_timeo
     if (!endpt) {
         return -1;
     }
+
     ret = nc_server_ssh_set_auth_timeout(auth_timeout, endpt->opts.ssh);
+
     /* UNLOCK */
     pthread_rwlock_unlock(&server_opts.endpt_lock);
 
@@ -558,7 +585,9 @@ nc_server_ssh_ch_client_endpt_set_auth_timeout(const char *client_name, const ch
     if (!endpt) {
         return -1;
     }
+
     ret = nc_server_ssh_set_auth_timeout(auth_timeout, endpt->opts.ssh);
+
     /* UNLOCK */
     nc_server_ch_client_unlock(client);
 
@@ -566,9 +595,10 @@ nc_server_ssh_ch_client_endpt_set_auth_timeout(const char *client_name, const ch
 }
 
 static int
-_nc_server_ssh_add_authkey(const char *pubkey_path, const char *pubkey_base64, NC_SSH_KEY_TYPE type,
-                          const char *username)
+_nc_server_ssh_add_authkey(const char *pubkey_path, const char *pubkey_base64, NC_SSH_KEY_TYPE type, const char *username)
 {
+    int ret = 0;
+
     /* LOCK */
     pthread_mutex_lock(&server_opts.authkey_lock);
 
@@ -576,17 +606,18 @@ _nc_server_ssh_add_authkey(const char *pubkey_path, const char *pubkey_base64, N
     server_opts.authkeys = nc_realloc(server_opts.authkeys, server_opts.authkey_count * sizeof *server_opts.authkeys);
     if (!server_opts.authkeys) {
         ERRMEM;
-        return -1;
+        ret = -1;
+        goto cleanup;
     }
-    server_opts.authkeys[server_opts.authkey_count - 1].path = lydict_insert(server_opts.ctx, pubkey_path, 0);
-    server_opts.authkeys[server_opts.authkey_count - 1].base64 = lydict_insert(server_opts.ctx, pubkey_base64, 0);
+    lydict_insert(server_opts.ctx, pubkey_path, 0, &server_opts.authkeys[server_opts.authkey_count - 1].path);
+    lydict_insert(server_opts.ctx, pubkey_base64, 0, &server_opts.authkeys[server_opts.authkey_count - 1].base64);
     server_opts.authkeys[server_opts.authkey_count - 1].type = type;
-    server_opts.authkeys[server_opts.authkey_count - 1].username = lydict_insert(server_opts.ctx, username, 0);
+    lydict_insert(server_opts.ctx, username, 0, &server_opts.authkeys[server_opts.authkey_count - 1].username);
 
+cleanup:
     /* UNLOCK */
     pthread_mutex_unlock(&server_opts.authkey_lock);
-
-    return 0;
+    return ret;
 }
 
 API int
@@ -622,7 +653,7 @@ nc_server_ssh_add_authkey(const char *pubkey_base64, NC_SSH_KEY_TYPE type, const
 
 API int
 nc_server_ssh_del_authkey(const char *pubkey_path, const char *pubkey_base64, NC_SSH_KEY_TYPE type,
-                          const char *username)
+        const char *username)
 {
     uint32_t i;
     int ret = -1;
@@ -643,10 +674,10 @@ nc_server_ssh_del_authkey(const char *pubkey_path, const char *pubkey_base64, NC
         server_opts.authkey_count = 0;
     } else {
         for (i = 0; i < server_opts.authkey_count; ++i) {
-            if ((!pubkey_path || !strcmp(server_opts.authkeys[i].path, pubkey_path))
-                    && (!pubkey_base64 || !strcmp(server_opts.authkeys[i].base64, pubkey_base64))
-                    && (!type || (server_opts.authkeys[i].type == type))
-                    && (!username || !strcmp(server_opts.authkeys[i].username, username))) {
+            if ((!pubkey_path || !strcmp(server_opts.authkeys[i].path, pubkey_path)) &&
+                    (!pubkey_base64 || !strcmp(server_opts.authkeys[i].base64, pubkey_base64)) &&
+                    (!type || (server_opts.authkeys[i].type == type)) &&
+                    (!username || !strcmp(server_opts.authkeys[i].username, username))) {
                 lydict_remove(server_opts.ctx, server_opts.authkeys[i].path);
                 lydict_remove(server_opts.ctx, server_opts.authkeys[i].base64);
                 lydict_remove(server_opts.ctx, server_opts.authkeys[i].username);
@@ -654,7 +685,7 @@ nc_server_ssh_del_authkey(const char *pubkey_path, const char *pubkey_base64, NC
                 --server_opts.authkey_count;
                 if (i < server_opts.authkey_count) {
                     memcpy(&server_opts.authkeys[i], &server_opts.authkeys[server_opts.authkey_count],
-                           sizeof *server_opts.authkeys);
+                            sizeof *server_opts.authkeys);
                 } else if (!server_opts.authkey_count) {
                     free(server_opts.authkeys);
                     server_opts.authkeys = NULL;
@@ -677,32 +708,98 @@ nc_server_ssh_clear_opts(struct nc_server_ssh_opts *opts)
     nc_server_ssh_del_hostkey(NULL, -1, opts);
 }
 
+#ifdef HAVE_SHADOW
+
+static struct passwd *
+auth_password_getpwnam(const char *username, struct passwd *pwd_buf, char **buf, size_t *buf_size)
+{
+    struct passwd *pwd = NULL;
+    char *mem;
+    int r = 0;
+
+    do {
+        r = getpwnam_r(username, pwd_buf, *buf, *buf_size, &pwd);
+        if (pwd) {
+            /* entry found */
+            break;
+        }
+
+        if (r == ERANGE) {
+            /* small buffer, enlarge */
+            *buf_size <<= 2;
+            mem = realloc(*buf, *buf_size);
+            if (!mem) {
+                ERRMEM;
+                return NULL;
+            }
+            *buf = mem;
+        }
+    } while (r == ERANGE);
+
+    return pwd;
+}
+
+static struct spwd *
+auth_password_getspnam(const char *username, struct spwd *spwd_buf, char **buf, size_t *buf_size)
+{
+    struct spwd *spwd = NULL;
+    char *mem;
+    int r = 0;
+
+    do {
+# ifndef __QNXNTO__
+        r = getspnam_r(username, spwd_buf, *buf, *buf_size, &spwd);
+# else
+        spwd = getspnam_r(username, spwd_buf, *buf, *buf_size);
+# endif
+        if (spwd) {
+            /* entry found */
+            break;
+        }
+
+        if (r == ERANGE) {
+            /* small buffer, enlarge */
+            *buf_size <<= 2;
+            mem = realloc(*buf, *buf_size);
+            if (!mem) {
+                ERRMEM;
+                return NULL;
+            }
+            *buf = mem;
+        }
+    } while (r == ERANGE);
+
+    return spwd;
+}
+
 static char *
 auth_password_get_pwd_hash(const char *username)
 {
-#ifdef HAVE_SHADOW
     struct passwd *pwd, pwd_buf;
     struct spwd *spwd, spwd_buf;
-    char *pass_hash = NULL, buf[256];
+    char *pass_hash = NULL, *buf = NULL;
+    size_t buf_size = 256;
 
-    getpwnam_r(username, &pwd_buf, buf, 256, &pwd);
+    buf = malloc(buf_size);
+    if (!buf) {
+        ERRMEM;
+        goto error;
+    }
+
+    pwd = auth_password_getpwnam(username, &pwd_buf, &buf, &buf_size);
     if (!pwd) {
-        VRB("User \"%s\" not found locally.", username);
-        return NULL;
+        VRB(NULL, "User \"%s\" not found locally.", username);
+        goto error;
     }
 
     if (!strcmp(pwd->pw_passwd, "x")) {
-# ifndef __QNXNTO__
-        getspnam_r(username, &spwd_buf, buf, 256, &spwd);
-# else
-        spwd = getspnam_r(username, &spwd_buf, buf, 256);
-# endif
+        spwd = auth_password_getspnam(username, &spwd_buf, &buf, &buf_size);
         if (!spwd) {
-            VRB("Failed to retrieve the shadow entry for \"%s\".", username);
-            return NULL;
+            VRB(NULL, "Failed to retrieve the shadow entry for \"%s\".", username);
+            goto error;
         } else if ((spwd->sp_expire > -1) && (spwd->sp_expire <= (time(NULL) / (60 * 60 * 24)))) {
-            WRN("User \"%s\" account has expired.", username);
-            return NULL;
+            WRN(NULL, "User \"%s\" account has expired.", username);
+            goto error;
         }
 
         pass_hash = spwd->sp_pwdp;
@@ -711,37 +808,52 @@ auth_password_get_pwd_hash(const char *username)
     }
 
     if (!pass_hash) {
-        ERR("No password could be retrieved for \"%s\".", username);
-        return NULL;
+        ERR(NULL, "No password could be retrieved for \"%s\".", username);
+        goto error;
     }
 
     /* check the hash structure for special meaning */
     if (!strcmp(pass_hash, "*") || !strcmp(pass_hash, "!")) {
-        VRB("User \"%s\" is not allowed to authenticate using a password.", username);
-        return NULL;
+        VRB(NULL, "User \"%s\" is not allowed to authenticate using a password.", username);
+        goto error;
     }
     if (!strcmp(pass_hash, "*NP*")) {
-        VRB("Retrieving password for \"%s\" from a NIS+ server not supported.", username);
-        return NULL;
+        VRB(NULL, "Retrieving password for \"%s\" from a NIS+ server not supported.", username);
+        goto error;
     }
 
-    return strdup(pass_hash);
-#else
-    return strdup("");
-#endif
+    pass_hash = strdup(pass_hash);
+    free(buf);
+    return pass_hash;
+
+error:
+    free(buf);
+    return NULL;
 }
 
+#else
+
+static char *
+auth_password_get_pwd_hash(const char *username)
+{
+    (void)username;
+    return strdup("");
+}
+
+#endif
+
 static int
 auth_password_compare_pwd(const char *pass_hash, const char *pass_clear)
 {
     char *new_pass_hash;
-#if defined(HAVE_CRYPT_R)
+
+#if defined (HAVE_CRYPT_R)
     struct crypt_data cdata;
 #endif
 
     if (!pass_hash[0]) {
         if (!pass_clear[0]) {
-            WRN("User authentication successful with an empty password!");
+            WRN(NULL, "User authentication successful with an empty password!");
             return 0;
         } else {
             /* the user did now know he does not need any password,
@@ -750,7 +862,7 @@ auth_password_compare_pwd(const char *pass_hash, const char *pass_clear)
         }
     }
 
-#if defined(HAVE_CRYPT_R)
+#if defined (HAVE_CRYPT_R)
     cdata.initialized = 0;
     new_pass_hash = crypt_r(pass_clear, pass_hash, &cdata);
 #else
@@ -784,11 +896,12 @@ nc_sshcb_auth_password(struct nc_session *session, ssh_message msg)
 
     if (!auth_ret) {
         session->flags |= NC_SESSION_SSH_AUTHENTICATED;
-        VRB("User \"%s\" authenticated.", session->username);
+        VRB(session, "User \"%s\" authenticated.", session->username);
         ssh_message_auth_reply_success(msg, 0);
     } else {
         ++session->opts.server.ssh_auth_attempts;
-        VRB("Failed user \"%s\" authentication attempt (#%d).", session->username, session->opts.server.ssh_auth_attempts);
+        VRB(session, "Failed user \"%s\" authentication attempt (#%d).", session->username,
+                session->opts.server.ssh_auth_attempts);
         ssh_message_reply_default(msg);
     }
 }
@@ -830,11 +943,12 @@ nc_sshcb_auth_kbdint(struct nc_session *session, ssh_message msg)
     /* Authenticate message based on outcome */
     if (!auth_ret) {
         session->flags |= NC_SESSION_SSH_AUTHENTICATED;
-        VRB("User \"%s\" authenticated.", session->username);
+        VRB(session, "User \"%s\" authenticated.", session->username);
         ssh_message_auth_reply_success(msg, 0);
     } else {
         ++session->opts.server.ssh_auth_attempts;
-        VRB("Failed user \"%s\" authentication attempt (#%d).", session->username, session->opts.server.ssh_auth_attempts);
+        VRB(session, "Failed user \"%s\" authentication attempt (#%d).", session->username,
+                session->opts.server.ssh_auth_attempts);
         ssh_message_reply_default(msg);
     }
 }
@@ -867,10 +981,10 @@ auth_pubkey_compare_key(ssh_key key)
         }
 
         if (ret == SSH_EOF) {
-            WRN("Failed to import a public key of \"%s\" (File access problem).", server_opts.authkeys[i].username);
+            WRN(NULL, "Failed to import a public key of \"%s\" (File access problem).", server_opts.authkeys[i].username);
             continue;
         } else if (ret == SSH_ERROR) {
-            WRN("Failed to import a public key of \"%s\" (SSH error).", server_opts.authkeys[i].username);
+            WRN(NULL, "Failed to import a public key of \"%s\" (SSH error).", server_opts.authkeys[i].username);
             continue;
         }
 
@@ -904,17 +1018,17 @@ nc_sshcb_auth_pubkey(struct nc_session *session, ssh_message msg)
         }
     } else {
         if ((username = auth_pubkey_compare_key(ssh_message_auth_pubkey(msg))) == NULL) {
-            VRB("User \"%s\" tried to use an unknown (unauthorized) public key.", session->username);
+            VRB(session, "User \"%s\" tried to use an unknown (unauthorized) public key.", session->username);
             goto fail;
         } else if (strcmp(session->username, username)) {
-            VRB("User \"%s\" is not the username identified with the presented public key.", session->username);
+            VRB(session, "User \"%s\" is not the username identified with the presented public key.", session->username);
             goto fail;
         }
     }
 
     signature_state = ssh_message_auth_publickey_state(msg);
     if (signature_state == SSH_PUBLICKEY_STATE_VALID) {
-        VRB("User \"%s\" authenticated.", session->username);
+        VRB(session, "User \"%s\" authenticated.", session->username);
         session->flags |= NC_SESSION_SSH_AUTHENTICATED;
         ssh_message_auth_reply_success(msg, 0);
     } else if (signature_state == SSH_PUBLICKEY_STATE_NONE) {
@@ -926,7 +1040,8 @@ nc_sshcb_auth_pubkey(struct nc_session *session, ssh_message msg)
 
 fail:
     ++session->opts.server.ssh_auth_attempts;
-    VRB("Failed user \"%s\" authentication attempt (#%d).", session->username, session->opts.server.ssh_auth_attempts);
+    VRB(session, "Failed user \"%s\" authentication attempt (#%d).", session->username,
+            session->opts.server.ssh_auth_attempts);
     ssh_message_reply_default(msg);
 }
 
@@ -943,16 +1058,16 @@ nc_sshcb_channel_open(struct nc_session *session, ssh_message msg)
         }
         chan = ssh_message_channel_request_open_reply_accept(msg);
         if (!chan) {
-            ERR("Failed to create a new SSH channel.");
+            ERR(session, "Failed to create a new SSH channel.");
             return -1;
         }
         session->ti.libssh.channel = chan;
 
-    /* additional channel request */
+        /* additional channel request */
     } else {
         chan = ssh_message_channel_request_open_reply_accept(msg);
         if (!chan) {
-            ERR("Session %u: failed to create a new SSH channel.", session->id);
+            ERR(session, "Session %u: failed to create a new SSH channel.", session->id);
             return -1;
         }
         /* channel was created and libssh stored it internally in the ssh_session structure, good enough */
@@ -967,7 +1082,7 @@ nc_sshcb_channel_subsystem(struct nc_session *session, ssh_channel channel, cons
     struct nc_session *new_session;
 
     if (strcmp(subsystem, "netconf")) {
-        WRN("Received an unknown subsystem \"%s\" request.", subsystem);
+        WRN(session, "Received an unknown subsystem \"%s\" request.", subsystem);
         return -1;
     }
 
@@ -978,7 +1093,7 @@ nc_sshcb_channel_subsystem(struct nc_session *session, ssh_channel channel, cons
             return -1;
         }
         if (session->flags & NC_SESSION_SSH_SUBSYS_NETCONF) {
-            ERR("Session %u: subsystem \"netconf\" requested for the second time.", session->id);
+            ERR(session, "Subsystem \"netconf\" requested for the second time.");
             return -1;
         }
 
@@ -1004,8 +1119,8 @@ nc_sshcb_channel_subsystem(struct nc_session *session, ssh_channel channel, cons
         new_session->io_lock = session->io_lock;
         new_session->ti.libssh.channel = channel;
         new_session->ti.libssh.session = session->ti.libssh.session;
-        new_session->username = lydict_insert(server_opts.ctx, session->username, 0);
-        new_session->host = lydict_insert(server_opts.ctx, session->host, 0);
+        lydict_insert(server_opts.ctx, session->username, 0, &new_session->username);
+        lydict_insert(server_opts.ctx, session->host, 0, &new_session->host);
         new_session->port = session->port;
         new_session->ctx = server_opts.ctx;
         new_session->flags = NC_SESSION_SSH_AUTHENTICATED | NC_SESSION_SSH_SUBSYS_NETCONF | NC_SESSION_SHAREDCTX;
@@ -1065,7 +1180,7 @@ nc_sshcb_msg(ssh_session UNUSED(sshsession), ssh_message msg, void *data)
             str_subtype = "channel-x11";
             break;
         case SSH_CHANNEL_UNKNOWN:
-            /* fallthrough */
+        /* fallthrough */
         default:
             str_subtype = "unknown";
             break;
@@ -1097,7 +1212,7 @@ nc_sshcb_msg(ssh_session UNUSED(sshsession), ssh_message msg, void *data)
             str_subtype = "x11";
             break;
         case SSH_CHANNEL_REQUEST_UNKNOWN:
-            /* fallthrough */
+        /* fallthrough */
         default:
             str_subtype = "unknown";
             break;
@@ -1119,7 +1234,7 @@ nc_sshcb_msg(ssh_session UNUSED(sshsession), ssh_message msg, void *data)
             str_subtype = "cancel-tcpip-forward";
             break;
         case SSH_GLOBAL_REQUEST_UNKNOWN:
-            /* fallthrough */
+        /* fallthrough */
         default:
             str_subtype = "unknown";
             break;
@@ -1132,12 +1247,12 @@ nc_sshcb_msg(ssh_session UNUSED(sshsession), ssh_message msg, void *data)
         break;
     }
 
-    VRB("Received an SSH message \"%s\" of subtype \"%s\".", str_type, str_subtype);
+    VRB(session, "Received an SSH message \"%s\" of subtype \"%s\".", str_type, str_subtype);
     if (!session || (session->status == NC_STATUS_CLOSING) || (session->status == NC_STATUS_INVALID)) {
         /* "valid" situation if, for example, receiving some auth or channel request timeouted,
          * but we got it now, during session free */
-        VRB("SSH message arrived on a %s session, the request will be denied.",
-            (session && session->status == NC_STATUS_CLOSING ? "closing" : "invalid"));
+        VRB(session, "SSH message arrived on a %s session, the request will be denied.",
+                (session && session->status == NC_STATUS_CLOSING ? "closing" : "invalid"));
         ssh_message_reply_default(msg);
         return 0;
     }
@@ -1148,7 +1263,7 @@ nc_sshcb_msg(ssh_session UNUSED(sshsession), ssh_message msg, void *data)
      */
     if (type == SSH_REQUEST_AUTH) {
         if (session->flags & NC_SESSION_SSH_AUTHENTICATED) {
-            ERR("User \"%s\" authenticated, but requested another authentication.", session->username);
+            ERR(session, "User \"%s\" authenticated, but requested another authentication.", session->username);
             ssh_message_reply_default(msg);
             return 0;
         }
@@ -1157,14 +1272,14 @@ nc_sshcb_msg(ssh_session UNUSED(sshsession), ssh_message msg, void *data)
         username = ssh_message_auth_user(msg);
         if (!session->username) {
             if (!username) {
-                ERR("Denying an auth request without a username.");
+                ERR(session, "Denying an auth request without a username.");
                 return 1;
             }
 
-            session->username = lydict_insert(server_opts.ctx, username, 0);
+            lydict_insert(server_opts.ctx, username, 0, &session->username);
         } else if (username) {
             if (strcmp(username, session->username)) {
-                ERR("User \"%s\" changed its username to \"%s\".", session->username, username);
+                ERR(session, "User \"%s\" changed its username to \"%s\".", session->username, username);
                 session->status = NC_STATUS_INVALID;
                 session->term_reason = NC_SESSION_TERM_OTHER;
                 return 1;
@@ -1217,14 +1332,14 @@ nc_open_netconf_channel(struct nc_session *session, int timeout)
      * created if timeout == 0 (it takes 2 messages, channel-open, subsystem-request) */
     if (!timeout) {
         if (!nc_session_is_connected(session)) {
-            ERR("Communication socket unexpectedly closed (libssh).");
+            ERR(session, "Communication socket unexpectedly closed (libssh).");
             return -1;
         }
 
         ret = ssh_execute_message_callbacks(session->ti.libssh.session);
         if (ret != SSH_OK) {
-            ERR("Failed to receive SSH messages on a session (%s).",
-                ssh_get_error(session->ti.libssh.session));
+            ERR(session, "Failed to receive SSH messages on a session (%s).",
+                    ssh_get_error(session->ti.libssh.session));
             return -1;
         }
 
@@ -1234,8 +1349,8 @@ nc_open_netconf_channel(struct nc_session *session, int timeout)
 
         ret = ssh_execute_message_callbacks(session->ti.libssh.session);
         if (ret != SSH_OK) {
-            ERR("Failed to receive SSH messages on a session (%s).",
-                ssh_get_error(session->ti.libssh.session));
+            ERR(session, "Failed to receive SSH messages on a session (%s).",
+                    ssh_get_error(session->ti.libssh.session));
             return -1;
         }
 
@@ -1253,14 +1368,14 @@ nc_open_netconf_channel(struct nc_session *session, int timeout)
     }
     while (1) {
         if (!nc_session_is_connected(session)) {
-            ERR("Communication socket unexpectedly closed (libssh).");
+            ERR(session, "Communication socket unexpectedly closed (libssh).");
             return -1;
         }
 
         ret = ssh_execute_message_callbacks(session->ti.libssh.session);
         if (ret != SSH_OK) {
-            ERR("Failed to receive SSH messages on a session (%s).",
-                ssh_get_error(session->ti.libssh.session));
+            ERR(session, "Failed to receive SSH messages on a session (%s).",
+                    ssh_get_error(session->ti.libssh.session));
             return -1;
         }
 
@@ -1273,7 +1388,7 @@ nc_open_netconf_channel(struct nc_session *session, int timeout)
             nc_gettimespec_mono(&ts_cur);
             if (nc_difftimespec(&ts_cur, &ts_timeout) < 1) {
                 /* timeout */
-                ERR("Failed to start \"netconf\" SSH subsystem for too long, disconnecting.");
+                ERR(session, "Failed to start \"netconf\" SSH subsystem for too long, disconnecting.");
                 break;
             }
         }
@@ -1291,21 +1406,21 @@ nc_ssh_bind_add_hostkeys(ssh_bind sbind, const char **hostkeys, uint8_t hostkey_
     NC_SSH_KEY_TYPE privkey_type;
 
     if (!server_opts.hostkey_clb) {
-        ERR("Callback for retrieving SSH host keys not set.");
+        ERR(NULL, "Callback for retrieving SSH host keys not set.");
         return -1;
     }
 
     for (i = 0; i < hostkey_count; ++i) {
         privkey_path = privkey_data = NULL;
         if (server_opts.hostkey_clb(hostkeys[i], server_opts.hostkey_data, &privkey_path, &privkey_data, &privkey_type)) {
-            ERR("Host key callback failed.");
+            ERR(NULL, "Host key callback failed.");
             return -1;
         }
 
         if (privkey_data) {
             privkey_path = base64der_key_to_tmp_file(privkey_data, nc_keytype2str(privkey_type));
             if (!privkey_path) {
-                ERR("Temporarily storing a host key into a file failed (%s).", strerror(errno));
+                ERR(NULL, "Temporarily storing a host key into a file failed (%s).", strerror(errno));
                 free(privkey_data);
                 return -1;
             }
@@ -1315,12 +1430,12 @@ nc_ssh_bind_add_hostkeys(ssh_bind sbind, const char **hostkeys, uint8_t hostkey_
 
         /* cleanup */
         if (privkey_data && unlink(privkey_path)) {
-            WRN("Removing a temporary host key file \"%s\" failed (%s).", privkey_path, strerror(errno));
+            WRN(NULL, "Removing a temporary host key file \"%s\" failed (%s).", privkey_path, strerror(errno));
         }
         free(privkey_data);
 
         if (ret != SSH_OK) {
-            ERR("Failed to set hostkey \"%s\" (%s).", hostkeys[i], privkey_path);
+            ERR(NULL, "Failed to set hostkey \"%s\" (%s).", hostkeys[i], privkey_path);
         }
         free(privkey_path);
 
@@ -1346,7 +1461,7 @@ nc_accept_ssh_session(struct nc_session *session, int sock, int timeout)
     session->ti_type = NC_TI_LIBSSH;
     session->ti.libssh.session = ssh_new();
     if (!session->ti.libssh.session) {
-        ERR("Failed to initialize a new SSH session.");
+        ERR(NULL, "Failed to initialize a new SSH session.");
         close(sock);
         return -1;
     }
@@ -1364,7 +1479,7 @@ nc_accept_ssh_session(struct nc_session *session, int sock, int timeout)
 
     sbind = ssh_bind_new();
     if (!sbind) {
-        ERR("Failed to create an SSH bind.");
+        ERR(session, "Failed to create an SSH bind.");
         close(sock);
         return -1;
     }
@@ -1380,7 +1495,7 @@ nc_accept_ssh_session(struct nc_session *session, int sock, int timeout)
     session->flags |= NC_SESSION_SSH_MSG_CB;
 
     if (ssh_bind_accept_fd(sbind, session->ti.libssh.session, sock) == SSH_ERROR) {
-        ERR("SSH failed to accept a new connection (%s).", ssh_get_error(sbind));
+        ERR(session, "SSH failed to accept a new connection (%s).", ssh_get_error(sbind));
         close(sock);
         ssh_bind_free(sbind);
         return -1;
@@ -1404,10 +1519,10 @@ nc_accept_ssh_session(struct nc_session *session, int sock, int timeout)
         }
     }
     if (ret == SSH_AGAIN) {
-        ERR("SSH key exchange timeout.");
+        ERR(session, "SSH key exchange timeout.");
         return 0;
     } else if (ret != SSH_OK) {
-        ERR("SSH key exchange error (%s).", ssh_get_error(session->ti.libssh.session));
+        ERR(session, "SSH key exchange error (%s).", ssh_get_error(session->ti.libssh.session));
         return -1;
     }
 
@@ -1418,13 +1533,13 @@ nc_accept_ssh_session(struct nc_session *session, int sock, int timeout)
     }
     while (1) {
         if (!nc_session_is_connected(session)) {
-            ERR("Communication SSH socket unexpectedly closed.");
+            ERR(session, "Communication SSH socket unexpectedly closed.");
             return -1;
         }
 
         if (ssh_execute_message_callbacks(session->ti.libssh.session) != SSH_OK) {
-            ERR("Failed to receive SSH messages on a session (%s).",
-                ssh_get_error(session->ti.libssh.session));
+            ERR(session, "Failed to receive SSH messages on a session (%s).",
+                    ssh_get_error(session->ti.libssh.session));
             return -1;
         }
 
@@ -1433,7 +1548,7 @@ nc_accept_ssh_session(struct nc_session *session, int sock, int timeout)
         }
 
         if (session->opts.server.ssh_auth_attempts >= opts->auth_attempts) {
-            ERR("Too many failed authentication attempts of user \"%s\".", session->username);
+            ERR(session, "Too many failed authentication attempts of user \"%s\".", session->username);
             return -1;
         }
 
@@ -1450,9 +1565,9 @@ nc_accept_ssh_session(struct nc_session *session, int sock, int timeout)
     if (!(session->flags & NC_SESSION_SSH_AUTHENTICATED)) {
         /* timeout */
         if (session->username) {
-            ERR("User \"%s\" failed to authenticate for too long, disconnecting.", session->username);
+            ERR(session, "User \"%s\" failed to authenticate for too long, disconnecting.", session->username);
         } else {
-            ERR("User failed to authenticate for too long, disconnecting.");
+            ERR(session, "User failed to authenticate for too long, disconnecting.");
         }
         return 0;
     }
@@ -1482,13 +1597,13 @@ nc_session_accept_ssh_channel(struct nc_session *orig_session, struct nc_session
         return NC_MSG_ERROR;
     }
 
-    if ((orig_session->status == NC_STATUS_RUNNING) && (orig_session->ti_type == NC_TI_LIBSSH)
-            && orig_session->ti.libssh.next) {
+    if ((orig_session->status == NC_STATUS_RUNNING) && (orig_session->ti_type == NC_TI_LIBSSH) &&
+            orig_session->ti.libssh.next) {
         for (new_session = orig_session->ti.libssh.next;
                 new_session != orig_session;
                 new_session = new_session->ti.libssh.next) {
-            if ((new_session->status == NC_STATUS_STARTING) && new_session->ti.libssh.channel
-                    && (new_session->flags & NC_SESSION_SSH_SUBSYS_NETCONF)) {
+            if ((new_session->status == NC_STATUS_STARTING) && new_session->ti.libssh.channel &&
+                    (new_session->flags & NC_SESSION_SSH_SUBSYS_NETCONF)) {
                 /* we found our session */
                 break;
             }
@@ -1499,12 +1614,12 @@ nc_session_accept_ssh_channel(struct nc_session *orig_session, struct nc_session
     }
 
     if (!new_session) {
-        ERR("Session does not have a NETCONF SSH channel ready.");
+        ERR(orig_session, "Session does not have a NETCONF SSH channel ready.");
         return NC_MSG_ERROR;
     }
 
     /* assign new SID atomically */
-    new_session->id = ATOMIC_INC(server_opts.new_session_id);
+    new_session->id = ATOMIC_INC_RELAXED(server_opts.new_session_id);
 
     /* NETCONF handshake */
     msgtype = nc_handshake_io(new_session);
@@ -1546,14 +1661,14 @@ nc_ps_accept_ssh_channel(struct nc_pollsession *ps, struct nc_session **session)
 
     for (i = 0; i < ps->session_count; ++i) {
         cur_session = ps->sessions[i]->session;
-        if ((cur_session->status == NC_STATUS_RUNNING) && (cur_session->ti_type == NC_TI_LIBSSH)
-                && cur_session->ti.libssh.next) {
+        if ((cur_session->status == NC_STATUS_RUNNING) && (cur_session->ti_type == NC_TI_LIBSSH) &&
+                cur_session->ti.libssh.next) {
             /* an SSH session with more channels */
             for (new_session = cur_session->ti.libssh.next;
                     new_session != cur_session;
                     new_session = new_session->ti.libssh.next) {
-                if ((new_session->status == NC_STATUS_STARTING) && new_session->ti.libssh.channel
-                        && (new_session->flags & NC_SESSION_SSH_SUBSYS_NETCONF)) {
+                if ((new_session->status == NC_STATUS_STARTING) && new_session->ti.libssh.channel &&
+                        (new_session->flags & NC_SESSION_SSH_SUBSYS_NETCONF)) {
                     /* we found our session */
                     break;
                 }
@@ -1570,12 +1685,12 @@ nc_ps_accept_ssh_channel(struct nc_pollsession *ps, struct nc_session **session)
     nc_ps_unlock(ps, q_id, __func__);
 
     if (!new_session) {
-        ERR("No session with a NETCONF SSH channel ready was found.");
+        ERR(NULL, "No session with a NETCONF SSH channel ready was found.");
         return NC_MSG_ERROR;
     }
 
     /* assign new SID atomically */
-    new_session->id = ATOMIC_INC(server_opts.new_session_id);
+    new_session->id = ATOMIC_INC_RELAXED(server_opts.new_session_id);
 
     /* NETCONF handshake */
     msgtype = nc_handshake_io(new_session);
