@@ -20,25 +20,26 @@
 #include <stddef.h>
 #include <stdint.h>
 #include <stdlib.h>
-#include <unistd.h>
 #include <string.h>
+#include <sys/socket.h>
 #include <sys/stat.h>
 #include <sys/types.h>
-#include <sys/socket.h>
+#include <unistd.h>
 
 #include <cmocka.h>
 #include <libyang/libyang.h>
 
-#include <session_client.h>
-#include <session_server.h>
-#include <session_p.h>
 #include <messages_p.h>
+#include <session_client.h>
+#include <session_p.h>
+#include <session_server.h>
 #include "tests/config.h"
 
 struct nc_session *server_session;
 struct nc_session *client_session;
 struct ly_ctx *ctx;
 pthread_mutex_t state_lock = PTHREAD_MUTEX_INITIALIZER;
+pthread_barrier_t barrier;
 int glob_state;
 
 struct nc_server_reply *
@@ -58,8 +59,7 @@ my_getconfig_rpc_clb(struct lyd_node *rpc, struct nc_session *session)
     assert_string_equal(rpc->schema->name, "get-config");
     assert_ptr_equal(session, server_session);
 
-    data = lyd_new_path(NULL, session->ctx, "/ietf-netconf:get-config/data", NULL, LYD_ANYDATA_CONSTSTRING,
-                        LYD_PATH_OPT_OUTPUT);
+    lyd_new_path(NULL, session->ctx, "/ietf-netconf:get-config/data", NULL, LYD_NEW_PATH_OUTPUT, &data);
     assert_non_null(data);
 
     return nc_server_reply_data(data, NC_WD_EXPLICIT, NC_PARAMTYPE_FREE);
@@ -156,9 +156,11 @@ teardown_sessions(void **state)
     (void)state;
 
     close(server_session->ti.fd.in);
+    server_session->ti.fd.in = -1;
     nc_session_free(server_session, NULL);
 
     close(client_session->ti.fd.in);
+    client_session->ti.fd.in = -1;
     nc_session_free(client_session, NULL);
 
     return 0;
@@ -171,7 +173,7 @@ test_send_recv_ok(void)
     uint64_t msgid;
     NC_MSG_TYPE msgtype;
     struct nc_rpc *rpc;
-    struct nc_reply *reply;
+    struct lyd_node *envp, *op;
     struct nc_pollsession *ps;
 
     /* client RPC */
@@ -193,12 +195,13 @@ test_send_recv_ok(void)
     nc_ps_free(ps);
 
     /* client reply */
-    msgtype = nc_recv_reply(client_session, rpc, msgid, 0, 0, &reply);
+    msgtype = nc_recv_reply(client_session, rpc, msgid, 0, &envp, &op);
     assert_int_equal(msgtype, NC_MSG_REPLY);
 
     nc_rpc_free(rpc);
-    assert_int_equal(reply->type, NC_RPL_OK);
-    nc_reply_free(reply);
+    assert_null(op);
+    assert_string_equal(LYD_NAME(lyd_child(envp)), "ok");
+    lyd_free_tree(envp);
 }
 
 static void
@@ -230,7 +233,7 @@ test_send_recv_error(void)
     uint64_t msgid;
     NC_MSG_TYPE msgtype;
     struct nc_rpc *rpc;
-    struct nc_reply *reply;
+    struct lyd_node *envp, *op, *node;
     struct nc_pollsession *ps;
 
     /* client RPC */
@@ -252,13 +255,16 @@ test_send_recv_error(void)
     nc_ps_free(ps);
 
     /* client reply */
-    msgtype = nc_recv_reply(client_session, rpc, msgid, 0, 0, &reply);
+    msgtype = nc_recv_reply(client_session, rpc, msgid, 0, &envp, &op);
     assert_int_equal(msgtype, NC_MSG_REPLY);
 
     nc_rpc_free(rpc);
-    assert_int_equal(reply->type, NC_RPL_ERROR);
-    assert_string_equal(((struct nc_reply_error *)reply)->err->tag, "operation-not-supported");
-    nc_reply_free(reply);
+    assert_string_equal(LYD_NAME(lyd_child(envp)), "rpc-error");
+    lyd_find_sibling_opaq_next(lyd_child(lyd_child(envp)), "error-tag", &node);
+    assert_non_null(node);
+    assert_string_equal(((struct lyd_node_opaq *)node)->value, "operation-not-supported");
+    lyd_free_tree(envp);
+    assert_null(op);
 }
 
 static void
@@ -290,7 +296,7 @@ test_send_recv_data(void)
     uint64_t msgid;
     NC_MSG_TYPE msgtype;
     struct nc_rpc *rpc;
-    struct nc_reply *reply;
+    struct lyd_node *envp, *op;
     struct nc_pollsession *ps;
 
     /* client RPC */
@@ -312,12 +318,14 @@ test_send_recv_data(void)
     nc_ps_free(ps);
 
     /* client reply */
-    msgtype = nc_recv_reply(client_session, rpc, msgid, 0, 0, &reply);
+    msgtype = nc_recv_reply(client_session, rpc, msgid, 0, &envp, &op);
     assert_int_equal(msgtype, NC_MSG_REPLY);
 
     nc_rpc_free(rpc);
-    assert_int_equal(reply->type, NC_RPL_DATA);
-    nc_reply_free(reply);
+    assert_non_null(envp);
+    lyd_free_tree(envp);
+    assert_non_null(op);
+    lyd_free_tree(op);
 }
 
 static void
@@ -342,30 +350,15 @@ test_send_recv_data_11(void **state)
     test_send_recv_data();
 }
 
-static void
-test_notif_clb(struct nc_session *session, const struct nc_notif *notif)
-{
-    assert_ptr_equal(session, client_session);
-    assert_string_equal(notif->tree->schema->name, "notificationComplete");
-
-    /* client notification received, update state */
-    pthread_mutex_lock(&state_lock);
-    while (glob_state != 2) {
-        pthread_mutex_unlock(&state_lock);
-        usleep(1000);
-        pthread_mutex_lock(&state_lock);
-    }
-    glob_state = 3;
-    pthread_mutex_unlock(&state_lock);
-}
-
 static void *
 server_send_notif_thread(void *arg)
 {
     NC_MSG_TYPE msg_type;
     struct lyd_node *notif_tree;
     struct nc_server_notif *notif;
+    struct timespec ts;
     char *buf;
+
     (void)arg;
 
     /* wait for the RPC callback to be called */
@@ -377,35 +370,59 @@ server_send_notif_thread(void *arg)
     }
 
     /* create notif */
-    notif_tree = lyd_new_path(NULL, ctx, "/nc-notifications:notificationComplete", NULL, 0, 0);
+    lyd_new_path(NULL, ctx, "/nc-notifications:notificationComplete", NULL, 0, &notif_tree);
     assert_non_null(notif_tree);
-    buf = malloc(64);
-    assert_non_null(buf);
-    notif = nc_server_notif_new(notif_tree, nc_time2datetime(time(NULL), NULL, buf), NC_PARAMTYPE_FREE);
+    clock_gettime(CLOCK_REALTIME, &ts);
+    ly_time_ts2str(&ts, &buf);
+    notif = nc_server_notif_new(notif_tree, buf, NC_PARAMTYPE_FREE);
     assert_non_null(notif);
 
     /* send notif */
-    nc_session_set_notif_status(server_session, 1);
+    nc_session_inc_notif_status(server_session);
     msg_type = nc_server_notif_send(server_session, notif, 100);
     nc_server_notif_free(notif);
     assert_int_equal(msg_type, NC_MSG_NOTIF);
 
     /* update state */
     glob_state = 2;
+    pthread_barrier_wait(&barrier);
     pthread_mutex_unlock(&state_lock);
 
     return NULL;
 }
 
+static void *
+thread_recv_notif(void *arg)
+{
+    struct nc_session *session = (struct nc_session *)arg;
+    struct lyd_node *envp;
+    struct lyd_node *op;
+    NC_MSG_TYPE msgtype;
+
+    pthread_barrier_wait(&barrier);
+    msgtype = nc_recv_notif(session, 1000, &envp, &op);
+    assert_int_equal(msgtype, NC_MSG_NOTIF);
+    assert_string_equal(op->schema->name, "notificationComplete");
+
+    lyd_free_tree(envp);
+    lyd_free_tree(op);
+
+    pthread_mutex_lock(&state_lock);
+    glob_state = 3;
+    pthread_mutex_unlock(&state_lock);
+
+    return (void *)0;
+}
+
 static void
 test_send_recv_notif(void)
 {
     int ret;
-    pthread_t tid;
+    pthread_t tid[2];
     uint64_t msgid;
     NC_MSG_TYPE msgtype;
     struct nc_rpc *rpc;
-    struct nc_reply *reply;
+    struct lyd_node *envp, *op;
     struct nc_pollsession *ps;
 
     /* client RPC */
@@ -416,8 +433,7 @@ test_send_recv_notif(void)
     assert_int_equal(msgtype, NC_MSG_RPC);
 
     /* client subscription */
-    ret = nc_recv_notif_dispatch(client_session, test_notif_clb);
-    assert_int_equal(ret, 0);
+    pthread_create(&tid[0], NULL, thread_recv_notif, client_session);
 
     /* create server */
     ps = nc_ps_new();
@@ -428,7 +444,7 @@ test_send_recv_notif(void)
     pthread_mutex_lock(&state_lock);
     glob_state = 0;
     pthread_mutex_unlock(&state_lock);
-    ret = pthread_create(&tid, NULL, server_send_notif_thread, NULL);
+    ret = pthread_create(&tid[1], NULL, server_send_notif_thread, NULL);
     assert_int_equal(ret, 0);
 
     /* server blocked on RPC */
@@ -441,17 +457,20 @@ test_send_recv_notif(void)
     pthread_mutex_unlock(&state_lock);
 
     /* server finished */
-    ret = pthread_join(tid, NULL);
+    ret = 0;
+    ret |= pthread_join(tid[0], NULL);
+    ret |= pthread_join(tid[1], NULL);
     assert_int_equal(ret, 0);
     nc_ps_free(ps);
 
     /* client reply */
-    msgtype = nc_recv_reply(client_session, rpc, msgid, 0, 0, &reply);
+    msgtype = nc_recv_reply(client_session, rpc, msgid, 0, &envp, &op);
     assert_int_equal(msgtype, NC_MSG_REPLY);
-
     nc_rpc_free(rpc);
-    assert_int_equal(reply->type, NC_RPL_OK);
-    nc_reply_free(reply);
+
+    assert_string_equal(LYD_NAME(lyd_child(envp)), "ok");
+    lyd_free_tree(envp);
+    assert_null(op);
 }
 
 static void
@@ -481,36 +500,37 @@ main(void)
 {
     int ret;
     const struct lys_module *module;
-    const struct lys_node *node;
+    struct lysc_node *node;
+    const char *nc_features[] = {"candidate", NULL};
+
+    pthread_barrier_init(&barrier, NULL, 2);
 
     /* create ctx */
-    ctx = ly_ctx_new(TESTS_DIR"/data/modules", 0);
+    ly_ctx_new(TESTS_DIR "/data/modules", 0, &ctx);
     assert_non_null(ctx);
 
     /* load modules */
-    module = ly_ctx_load_module(ctx, "ietf-netconf-acm", NULL);
+    module = ly_ctx_load_module(ctx, "ietf-netconf-acm", NULL, NULL);
     assert_non_null(module);
 
-    module = ly_ctx_load_module(ctx, "ietf-netconf", NULL);
+    module = ly_ctx_load_module(ctx, "ietf-netconf", NULL, nc_features);
     assert_non_null(module);
-    ret = lys_features_enable(module, "candidate");
-    assert_int_equal(ret, 0);
 
-    module = ly_ctx_load_module(ctx, "nc-notifications", NULL);
+    module = ly_ctx_load_module(ctx, "nc-notifications", NULL, NULL);
     assert_non_null(module);
 
     /* set RPC callbacks */
-    node = ly_ctx_get_node(module->ctx, NULL, "/ietf-netconf:get", 0);
+    node = (struct lysc_node *)lys_find_path(module->ctx, NULL, "/ietf-netconf:get", 0);
     assert_non_null(node);
-    lys_set_private(node, my_get_rpc_clb);
+    node->priv = my_get_rpc_clb;
 
-    node = ly_ctx_get_node(module->ctx, NULL, "/ietf-netconf:get-config", 0);
+    node = (struct lysc_node *)lys_find_path(module->ctx, NULL, "/ietf-netconf:get-config", 0);
     assert_non_null(node);
-    lys_set_private(node, my_getconfig_rpc_clb);
+    node->priv = my_getconfig_rpc_clb;
 
-    node = ly_ctx_get_node(module->ctx, NULL, "/ietf-netconf:commit", 0);
+    node = (struct lysc_node *)lys_find_path(module->ctx, NULL, "/ietf-netconf:commit", 0);
     assert_non_null(node);
-    lys_set_private(node, my_commit_rpc_clb);
+    node->priv = my_commit_rpc_clb;
 
     nc_server_init(ctx);
 
@@ -528,7 +548,8 @@ main(void)
     ret = cmocka_run_group_tests(comm, NULL, NULL);
 
     nc_server_destroy();
-    ly_ctx_destroy(ctx, NULL);
+    ly_ctx_destroy(ctx);
+    pthread_barrier_destroy(&barrier);
 
     return ret;
 }
