@@ -20,17 +20,17 @@
 #include <stddef.h>
 #include <stdint.h>
 #include <stdlib.h>
-#include <unistd.h>
 #include <string.h>
+#include <sys/socket.h>
 #include <sys/stat.h>
 #include <sys/types.h>
-#include <sys/socket.h>
+#include <unistd.h>
 
 #include <cmocka.h>
 #include <libyang/libyang.h>
 
-#include "tests/config.h"
 #include <session_server.h>
+#include "tests/config.h"
 
 struct ly_ctx *ctx;
 
@@ -39,7 +39,7 @@ setup_server(void **state)
 {
     (void)state;
 
-    ctx = ly_ctx_new(NULL, 0);
+    ly_ctx_new(NULL, 0, &ctx);
     assert_non_null(ctx);
 
     nc_server_init(ctx);
@@ -53,7 +53,7 @@ teardown_server(void **state)
     (void)state;
 
     nc_server_destroy();
-    ly_ctx_destroy(ctx, NULL);
+    ly_ctx_destroy(ctx);
 
     return 0;
 }
@@ -73,4 +73,3 @@ main(void)
 
     return cmocka_run_group_tests(init_destroy, NULL, NULL);
 }
-
